/*
This program was designed and implemented by Silaghi, Marius-Calin.
The package sJavap was obtained from sJava (developed by Prof. Petitpierre)
The package platform was initially the semester project of Michel Galley in 2000
Copyright (c) 1998, 1999, 2000, 2001, 2002, 2004
*/
//-----------------------------------------------------------------------------
// Intelligent Agents Collaborating in Distributed Problem Solving

// Supervisors:   Marius Silaghi, Djamila Sam-Haroud
// Author:        Michel Galley

// File:          Protocol_BasicDS.java
// Version:       1.0
// Description:
//-----------------------------------------------------------------------------
// Interface:
//-----------------------------------------------------------------------------

package basic_ds;

import platform.*;
import csp.*;
import sjavap.*;
import java.io.*;

public class Protocol_BasicDS extends Protocol
{
    //Common variables:
    private Message_Command message = null;
    private boolean terminated = false;
    private int nbOfVariables = -1;

    // Varibles for the broker:
    private Broker broker;
    private int[] levelTable;
    private int source;

    // Variables for the agent:
    private Agent agent;

    SetOfVariables vset;
    Variable[] vars;
    SetOfConstraints cset;
    Constraint[] constr;
    SetOfDomains dset;
    Domain[] domains;

    int[] currentValues;
    int level;
    private boolean solving = false;

    /**************************
     * BROKER RELATED METHODS *
     *************************/

    public int _getMessage = 1;
    synchronized public void getMessage(Message message)
    {
	this.message = (Message_Command)message;
	Synch_Reply();
    }

    public int _terminate = 2;
    synchronized public void terminate()
    {
	terminated = true;
	Synch_Reply();
    }

    public String toString() { return "BasicDS"; }

    synchronized public void run()
    {

	initProtocol();
	System.out.println("@Protocol BasicDS started." + this);

	if(genericAgent instanceof Broker) {

	    if(agentIDs.length == 0) {
		System.out.println("No agent to solve problem! Leave.");
		return;
	    }

	    // BROKER RUN CODE:
	    broker = (Broker)genericAgent;

	    // Create the level table :
	    levelTable = new int[agentIDs.length];
	    for(int index=0; index<levelTable.length; index++)
		levelTable[index] = -1;
	    levelTable[0] = 0;

	    // Sends "start" message to each agent:
	    select.call(0, broker, broker._sendCommand); // first agent
	    select.accept(1, _terminate, 1);
	    Synch_Wait();
	    broker.sendCommand("start", null, agentIDs[0]);

	    for(int index=1; index<levelTable.length; index++) {// Other agents
		select.call(0, broker, broker._sendCommand);
		select.accept(1, _terminate, 1);
		Synch_Wait();
		broker.sendCommand("start", agentIDs[0], agentIDs[index]);
	    }

	    /**
	     * Main loop
	     */

	    Message_Command message;
	    String command;
	    source = 0;

	    while(!terminated) {

		select.accept(0, _terminate, 1);
		select.accept(1, _getMessage);

		switch(Synch_Wait()) {
		case 1:
		    processMessage();
		    message = null;
		}

		if(!terminated) {

		    // Checks if there is no solution:
		    terminated = true;
		    for(int index=0; index<levelTable.length; index++)
			//if(levelTable[index] != -1)
			    if(levelTable[index] != nbOfVariables) {
				terminated = false;
				break; // no global solution
			    }
		    if(terminated)
			System.out.println("No solution, leaving.");
		}
	    }

	} else {

	    // AGENT RUN CODE:

	    if(problem == null) {
		System.err.println("No problem loaded!");
		return;
	    }

   	    /**
	     * Main loop
	     */

	    agent = (Agent)genericAgent;
	    Message_Command message;
	    String command;
	    int reply;

	    vset = problem.getSetOfVariables();
	    vars = vset.getVariables();
	    cset = problem.getSetOfConstraints();
	    constr = cset.getConstraints();

	    nbOfVariables = problem.getVariablesNr();

	    currentValues = new int[vars.length];
	    int currentValue, nextValue = 0;
	    int currentVar = 0;
	    boolean backtrack;
	    level = -1;

	    boolean okValue;

	    while(!terminated) {

		select.accept(0, _terminate, 1);
		select.accept(1, _getMessage);
		if(solving)
		    select.waituntil(2, now()); // can't wait if solving

		reply = Synch_Wait();
		//System.out.println("Basic_DS: Solving ="+solving
		//		   +" Reply ="+reply);

		//System.out.println("SoD: "+genericAgent.getID()+": "+dset);

		switch(reply) {
		case 1:
		    processMessage();
		    message = null;
		}

		if(!solving)
		    continue;

// 		System.out.print("-----------------------------\nLevel="
// 				 +level+" Cur. variable="+currentVar
// 				 +"\nVariables:");
// 		for(int i=0; i<vars.length; i++) {
// 		    System.out.print(" "+vars[i].getName());
// 		    if(i==currentVar)
// 			System.out.print("*");
// 		}
// 		System.out.print("\tValues:   ");
// 		for(int i=0; i<currentValues.length; i++) {
// 		    System.out.print(" "+currentValues[i]);
// 		    if(i==currentVar)
// 			System.out.print("*");
// 		}
// 		System.out.println("\n"+dset);

		// SOLVING:

		// Checks if the value currentValues[currentVar]
		// satisfies all the constraints:
 		okValue = true;

		// Unary constraint(s):
		if(!cset.satisfiesConstraints(vars[currentVar].getName(),
					      currentValues[currentVar])) {
		    okValue = false;
		    //System.out.println("Var "+vars[currentVar].getName() +"="
		    //	       +currentValues[currentVar]+" : not ok");
		}
		else
		    // Binary constraints:
		    for(int i=0; i<currentVar; i++)
			if(!cset.satisfiesConstraints
			   (vars[i].getName(),
			    currentValues[i],
			    vars[currentVar].getName(),
			    currentValues[currentVar])) {

			    okValue = false;
			    break;
			}

		if(okValue) { // Current value ok:
		    //System.out.println("Good");
		    // check the next variable (if any)
		    if(currentVar + 1 == currentValues.length) {
			// found a solution
			//System.out.println("Solution");

			// Build solution string:
			String sol = "";
			for(int i=0; i<currentValues.length; i++)
			    sol += vars[i].getName()+"="
				+currentValues[i] + " ";

			// Send the solution to the broker
			select.call(0, agent, agent._sendCommand);
			select.accept(1, _terminate, 1);
			Synch_Wait();
			agent.sendCommand("solution", sol,
					  agent.getBrokerID());
			solving = false;
			terminated = true;
		    } else {
			// We take the next variable :
			// sets it to the first value in domain
			currentVar++;
			currentValues[currentVar]
			    = domains[currentVar].firstInDomain();

// 			System.out.println("\nVariables:");
// 			for(int i=0; i<vars.length; i++) {
// 			    System.out.print(" "+vars[i].getName());
// 			    if(i==currentVar)
// 				System.out.print("*");
// 			}
// 			System.out.print("\nValues:   ");
// 			for(int i=0; i<currentValues.length; i++) {
// 			    System.out.print(" "+currentValues[i]);
// 			    if(i==currentVar)
// 				System.out.print("*");
// 			}
		    }

		} else { // Current value not ok:
		    //System.out.println("Nogood");

		    // Consider the next value in domain;
		    // If there is no such variable, backtrack
		    // as many time as needed
		    do {

			// Remove the old value from the domain (if needed)
			if(currentVar == level) {
			    // remove the old value from the domain :
			    // it can't be in the solution
			    domains[currentVar].inDomain
				(currentValues[currentVar], false);
			    computeLevel();
			    //System.out.println("Remove from domain");
			}

			nextValue = domains[currentVar].nextInDomain
			    (currentValues[currentVar]);
			backtrack = (nextValue == currentValues[currentVar]);
			if(!backtrack) {
			    currentValues[currentVar] = nextValue;
			    break;
			}

			// No next value in domain: backtrack
			//System.out.println("Backtrack");
			currentValues[currentVar]
			    = domains[currentVar].firstInDomain(); // (opt)
			currentVar--;
			if(currentVar < level) {
			    // No solution : say that level = nb of var. -1
			    //System.out.println("No solution");
			    level = nbOfVariables;
			    sendLevel();
			    dset = null; domains = null;

			    // Get the new source
			    select.call(0, agent, agent._sendCommand);
			    select.accept(1, _terminate, 1);
			    Synch_Wait();
			    agent.sendCommand("getSource", null,
					      agent.getBrokerID());
			    solving = false;
			    break;
			}
		    } while(backtrack);
		}
	    }
	}
	System.out.println("@Protocol BasicDS ended.");
	cleanUpProtocol();
    }

    /**
     * Private methods
     */

    private void processMessage()
    {
	if(genericAgent instanceof Broker) {

	    // THE BROKER GETS A NEW MESSAGE:
	    String command = (String)message.getCommand();
	    Object arguments = message.getArguments();

	    // GETSOURCE
	    if(command.compareTo("getSource") == 0) {
		select.call(0, broker, broker._sendCommand);
		select.accept(1, _terminate, 1);
		Synch_Wait();

		// look for the smallest integer (!= -1) in the level
		// table, and send the ID of the corresponding agent
		// to the requester.
		Object[] s_and_l = new Object[2];
		s_and_l[0] = agentIDs[source];
		s_and_l[1] = new Integer(levelTable[source]);

		broker.sendCommand("source", s_and_l, message.getSenderID());
	    }
	    // MYLEVEL
	    else if(command.compareTo("myLevel") == 0) {
		// Find the index of the agent in the table
		for(int index = 0; index < levelTable.length; index++)
		    if(message.getSenderID().equals(agentIDs[index])) {
			// Agent found : put the new level in the table
			if(nbOfVariables == -1)
			    nbOfVariables
				= ((Integer[])arguments)[1].intValue();
			levelTable[index]
			    = ((Integer[])arguments)[0].intValue();
			break;
		    }

		// The table has changed : search the new source :
		for(int index = 0; index < levelTable.length; index++)
		    if(levelTable[index] != -1)
			if(levelTable[source] > levelTable[index])
			    source = index;
	    }
	    // SOLUTION
	    else if(command.compareTo("solution") == 0) {
		for(int index=0; index<agentIDs.length; index++) {
		    select.call(0, broker, broker._sendCommand);
		    select.accept(1, _terminate, 1);
		    Synch_Wait();
		    broker.sendCommand("terminate", null, agentIDs[index]);
		}
		terminated = true;
	    }
	    else
		System.out.println("Trash message received : "
				   +message);

	} else {

	    // THE AGENT GETS A NEW MESSAGE:
	    String command = (String)message.getCommand();
	    Object arguments = message.getArguments();


	    // START
	    if(command.compareTo("start") == 0) {
		if(arguments == null) {
		    // This agent is the source:
		    // solve local problem
		    dset = new SetOfDomains(vset);
		    updateDomains();
		    solving = true;
		} else {
		    select.call(0, agent, agent._sendCommand);
		    select.accept(1, _terminate, 1);
		    Synch_Wait();
		    agent.sendCommand("getProblem", new Integer(0),
				      (AgentID)arguments);
		}
	    }
	    else if(command.compareTo("source") == 0) {
		select.call(0, agent, agent._sendCommand);
		select.accept(1, _terminate, 1);
		Synch_Wait();
		agent.sendCommand("getProblem",
				  (Integer)((Object[])arguments)[1],
				  (AgentID)((Object[])arguments)[0]);
	    }

	    // GET PROBLEM
	    else if(command.compareTo("getProblem") == 0) {

		// Split the the range into two ranges:
		if(((Integer)arguments).intValue() < level || !solving) {
		    select.call(0, agent, agent._sendCommand);
		    select.accept(1, _terminate);
		    Synch_Wait();
		    agent.sendCommand("levelOutdated", null,
				      message.getSenderID());
		} else {

		    SetOfDomains newSet = (SetOfDomains)dset.clone();
		    Domain[] newDomains = newSet.getDomains();

		    // Find a value in the current level domain
		    int val = domains[level].firstInDomain();
		    if(val == currentValues[level])
			val = domains[level].nextInDomain(val);
		    if(val == currentValues[level]) {
			// Only one value in domain : Level <> real level
			System.err.println("Wrong level");
			System.exit(-1);
		    }

		    // remove it from agent's domain
		    domains[level].inDomain(val, false);

		    // and give it to the other agent
		    for(int i=vset.getRange(vars[level].getName()).getL();
			i<=vset.getRange(vars[level].getName()).getU(); i++)
			newDomains[level].inDomain(i, false);
		    newDomains[level].inDomain(val, true);

		    select.call(0, agent, agent._sendCommand);
		    select.accept(1, _terminate, 1);
		    Synch_Wait();
		    agent.sendCommand("setProblem", newSet,
				      message.getSenderID());
		    computeLevel();

		}
	    }

	    // SET PROBLEM
	    else if(command.compareTo("setProblem") == 0) {
		dset = (SetOfDomains)arguments;
		updateDomains();

		solving = true;
	    }

	    // LEVEL OUTDATED
	    else if(command.compareTo("levelOutdated") == 0) {
		// Sends a "getSource" to the broker:
		select.call(0, agent, agent._sendCommand);
		select.accept(1, _terminate, 1);
		Synch_Wait();
		agent.sendCommand("getSource", null, agent.getBrokerID());
	    }
	    else if(command.compareTo("terminate") == 0)
		terminated = true;
	}
    }

    private void updateDomains()
    {
	domains = dset.getDomains();

	for(int i=0; i<currentValues.length; i++)
	    currentValues[i] = domains[i].firstInDomain();
	computeLevel();
    }

    private void computeLevel()
    {
	int oldLevel = level;
	int i=0;
	while(domains[i].size() == 1) i++;
// 	if(domains[i].size() < 1) {
// 	    System.err.println("Unexpected values");
// 	    System.exit(-1);
// 	}

	//System.out.println("Levels: ("+i+")");
	//for(int j=0; j<domains.length; j++)
	//    System.out.print(domains[j].size());
	//System.out.println("");

	level = i;
	if(level != oldLevel)
	    sendLevel();
    }

    private void sendLevel()
    {
	select.call(0, agent, agent._sendCommand);
	select.accept(1, _terminate, 1);
	Synch_Wait();
	Object[] l_and_v = new Integer[2];
	l_and_v[0] = new Integer(level);
	l_and_v[1] = new Integer(nbOfVariables);
	agent.sendCommand("myLevel", l_and_v, agent.getBrokerID());
    }
}
