/*
This program was designed and implemented by Silaghi, Marius-Calin.
The package sJavap was obtained from sJava (developed by Prof. Petitpierre)
The package platform was initially the semester project of Michel Galley in 2000
Copyright (c) 1998, 1999, 2000, 2001, 2002, 2004
*/
//-----------------------------------------------------------------------------
// Intelligent Agents Collaborating in Distributed Problem Solving

// Supervisors:   Marius Silaghi, Djamila Sam-Haroud 
// Author:        Michel Galley

// File:          Domain.java
// Version:       1.0
// Description:   
//-----------------------------------------------------------------------------
// Interface:
//-----------------------------------------------------------------------------


package csp;
import java.util.*;
import java.io.*;

public class Domain implements Serializable, Cloneable
{
    private Range range;
    private boolean[] inDomain;
    
    public Domain(Range _range)
    {
	range = new Range(_range);
	inDomain = new boolean[range.getU()-range.getL()+1];
	
	for(int i=range.getL(); i<=range.getU(); i++)
	    inDomain[i-range.getL()] = true;
    }

    public Domain(Range _range, C1Representation constr)
    {
	this(_range);		
	int[] vect = constr.getMatrix();
	
	if(vect.length != inDomain.length) {
	    System.err.println("Constraint does not match");
	    System.exit(-1);
	}

	for(int i=0; i< vect.length; i++)
	    inDomain[i] = (vect[i] != 0);    
    }

    public boolean isInDomain(int value)
    {
	if(value < range.getL() || value > range.getU())
	    return false;
	return inDomain[value-range.getL()];
    }

    public void inDomain(int value, boolean in)
    {
	inDomain[value-range.getL()] = in;
    }

    public int firstInDomain()
    {
// 	System.out.println("Domain:");
// 	for(int i=0; i<inDomain.length; i++)
// 	    System.out.print(inDomain[i]+" ");
// 	System.out.println("");

	return nextInDomain(range.getL()-1);
    }

    public int nextInDomain(int value)
    {
	int index = value+1;

	//if(index < range.getU())
	//    index = range.getL();

	while(index >= range.getL() && index <= range.getU()) {
	    if(inDomain[index-range.getL()])
		return index;
	    index++;
	}

	return value;
    }

    public Object clone()
    {
	Domain clone = null;

	try { clone = (Domain)super.clone(); }
	catch(CloneNotSupportedException e) 
	{ e.printStackTrace(); System.exit(-1); }

	clone.inDomain = new boolean[inDomain.length];
	for(int i=0; i<inDomain.length; i++)
	    clone.inDomain[i] = inDomain[i]; 

	return clone;
    }

    public int size()
    {
	int size = 0;
	
	for(int index=0; index < inDomain.length; index++)
	    if(inDomain[index])
		size++;
	return size;
    }

    public String toString()
    {
	String str = "";
	boolean first = true;

	for(int i=range.getL(); i<=range.getU(); i++)
	    if(inDomain[i-range.getL()])
		if(first) {		
		    str += i;
		    first = false;
		} else
		    str += ","+i;
		
	return str;
    }
}
    
