/*
This program was designed and implemented by Silaghi, Marius-Calin.
The package sJavap was obtained from sJava (developed by Prof. Petitpierre)
The package platform was initially the semester project of Michel Galley in 2000
Copyright (c) 1998, 1999, 2000, 2001, 2002, 2004
*/
//-----------------------------------------------------------------------------
// Intelligent Agents Collaborating in Distributed Problem Solving

// Supervisors:   Marius Silaghi, Djamila Sam-Haroud
// Author:        Marius Silaghi

// File:          Problem.java
// Version:       1.0
// Description:
//-----------------------------------------------------------------------------
// Interface:
//-----------------------------------------------------------------------------

package csp;
import java.io.*;
import utils.*;

public class Problem implements Serializable{
    private SetOfConstraints cset;
    private SetOfVariables vset;
    public String variable;
    private String fname;
    public int workAmount=-1;//unlimited
    public boolean artificial=false;//unlimited
    public boolean mLastArtificial=false;
    public boolean mSplitter=false;
    public int mCoverSeedsNr=0;
    /**
     * Standard constructor
     */

    public Problem()
    {
	vset=new SetOfVariables(this);
	cset=new SetOfConstraints(this);
    }

    /**
     * Public methods
     */

    public void coverSeeds(int coverSeedsNr){
	if(coverSeedsNr==0)return;
	cset.coverSeeds(coverSeedsNr);
    }
    public int getVariablesNr() {
	return vset.getVariablesNr();
    }

    public SetOfVariables getSetOfVariables() {
	return vset;
    }
    public Variable[] getVariables() {
	return getSetOfVariables().getVariables();
    }
    public int getComplexity() {
	return cset.getComplexity();
    }
    public SetOfConstraints getSetOfConstraints() {
	return cset;
    }

    public boolean loadFile(File file) {
	String buf = "";
	RandomAccessFile fis=null;
	try {
	    fis= new RandomAccessFile(file, "r");
	    buf=Utils.getString(fis);
	}catch (Exception e){
	    System.err.println("Problem:loadFile:"+e);
	    e.printStackTrace();
	    try{
		fis.close();
	    }catch(java.io.IOException ev){
		System.err.println(ev);
		ev.printStackTrace();
	    }
	    return false;
	}
	if (buf.equals("COLOR")){
	    //System.out.println("Loading COLOR ");
	    colorLoader(file,fis,0);
	    try{
		fis.close();
	    }catch(Exception ev){
		System.err.println(ev);
		ev.printStackTrace();
	    }
	    return true;
	}
	if (buf.equals("COLOR_V1")){
	    //System.out.println("Loading COLOR_V1 ");
	    colorLoader(file,fis,1);
	    try{
		fis.close();
	    }catch(Exception ev){
		System.err.println(ev);
		ev.printStackTrace();
	    }
	    return true;
	}
	System.out.println("Unknown file Type:\""+buf+"\"");
	if(fis!=null)
	    try{
		fis.close();
	    }catch(Exception ev){
		System.err.println(ev);
		ev.printStackTrace();
	    }
	return false;
    }
    void cmp(String s1, String s2) {
	byte b1[]=s1.getBytes();
	byte b2[]=s2.getBytes();
	int k;
	System.out.println("Comparing "+s1+" to "+s2);
	for(k=0;k<b1.length; k++)
	    System.out.print(b1[k]+":");
	System.out.println("EOL");
	for(k=0;k<b2.length; k++)
	    System.out.print(b2[k]+":");
	System.out.println("EOL");
    }

    void colorLoader(File file, RandomAccessFile fis, int fileType) {
	int constraintsNr=Utils.getInt(fis);
	//System.out.println(constraintsNr +" Constraints");
	int k=0;
	for(;k<constraintsNr; k++) {
		int varNr=Utils.getInt(fis);
		cset.colorLoader(fis,varNr,file.getName()+"://"+k, fileType);
	    }
	fname=file.getName();
    }
    public String toString(){
	if(fname!=null) return fname+workAmount+" "+variable;
	return "Problem:"+workAmount+" "+variable;
    }
}
