/*
This program was designed and implemented by Silaghi, Marius-Calin.
The package sJavap was obtained from sJava (developed by Prof. Petitpierre)
The package platform was initially the semester project of Michel Galley in 2000
Copyright (c) 1998, 1999, 2000, 2001, 2002, 2004
*/
//-----------------------------------------------------------------------------
// Intelligent Agents Collaborating in Distributed Problem Solving

// Supervisors:   Marius Silaghi, Djamila Sam-Haroud 
// Authors:       Marius Silaghi, Michel Galley

// File:          SetOfConstraints.java
// Version:       1.0
// Description:
//-----------------------------------------------------------------------------
// Interface:
//-----------------------------------------------------------------------------

package csp;

import java.util.*;
import java.io.*;

public class SetOfConstraints implements Serializable{
    private Hashtable cons = new Hashtable();
    private Problem problem;

    public SetOfConstraints(Problem _problem) {
	problem=_problem;
    }

    public void coverSeeds(int coverSeedsNr){
	Iterator it=(Iterator)cons.keys();
	while(it.hasNext()){
	    Constraint c=(Constraint)cons.get(it.next());
	    c.getRepresentation().coverSeeds(coverSeedsNr);
	}
    }

    public Problem getProblem() {
	return problem;
    }

    public int getConstraintsNr() {
	return cons.size();
    }

    public Constraint[] getConstraints() {
	return (Constraint[])cons.values().toArray(new Constraint[0]);
    }

    public int getComplexity() {
	return getConstraintsNr();
    }

    public boolean satisfiesConstraints(String var, int val)
    {
	Constraint[] constrArray = getConstraints();
	boolean satisfiesOne = false;

	for(int i=0; i<constrArray.length; i++) {
	    if(constrArray[i].getRepresentation() 
	       instanceof C1Representation) {
		C1Representation c1 
		    = (C1Representation)constrArray[i].getRepresentation();
		try {
		    if(c1.satisfiesConstraint(var, val))
			satisfiesOne = true;
		    else
			return false;			
		} catch(FalseConstraintException e) {}
	    }
	}
	//return satisfiesOne;
	return true;
    }

    public boolean satisfiesConstraints(String var1, int val1, 
					String var2, int val2)
    {
	Constraint[] constrArray = getConstraints();
	boolean satisfiesOne = false;

	for(int i=0; i<constrArray.length; i++) {
	    if(constrArray[i].getRepresentation() 
	       instanceof C2Representation) {
		C2Representation c2 
		    = (C2Representation)constrArray[i].getRepresentation();
		try { 
		    if(c2.satisfiesConstraint(var1, val1, var2, val2))
			satisfiesOne = true;
		    else 
			return false;
		} catch(FalseConstraintException e) {}
		try {
		    if(c2.satisfiesConstraint(var2, val2, var1, val1))
			satisfiesOne = true;
		    else
			return false;
		} catch(FalseConstraintException e) {}
	    }
	}
	return true; //satisfiesOne;
    }

    public boolean colorLoader(RandomAccessFile fis,int varNr,
			       String key,int fileType) {
	Constraint c= new Constraint(this,varNr);
	//System.out.println(key);
	c.loadFile(fis,varNr,fileType);
	if(cons.put(key,c)!=null)
	    System.out.println("Constraint replaced");	    
	return true;
    }
}
