/*
This program was designed and implemented by Silaghi, Marius-Calin.
The package sJavap was obtained from sJava (developed by Prof. Petitpierre)
The package platform was initially the semester project of Michel Galley in 2000
Copyright (c) 1998, 1999, 2000, 2001, 2002, 2004
*/
package dcsp;
import java.util.*;
import csp.*;
import hc.*;
import utils.*;
import parameters.*;
import platform.*;

class LabelsManager{
    private int agentsNr, variablesNr;
    int consistentLevel;//, position
    private String variables[];
    private Range mHCk[];
    private LCNHashtable mHCks[][];
    private ILCN levelIntersection[][];
    private IOHashtable vIDs;
    private RHashtable localInstantiations[], addedInstantiations[];
    private ConflictSet addedVars=new ConflictSet();
    private ConflictSet linksToAdd;
    private Histories varHistory=new Histories();
    private Histories addedHistory=new Histories();
    private int proposalID[][];

    private int unknownID=-2;
    int[] basePID;
    private PAgent agent;

    void printStructures(){
	String s="consistentLevel="+consistentLevel+"\n";
	for(int v=0;v<variablesNr;v++){
	    s=s+variables[v]+" "+mHCk[v];
	}
	for(int a=0;a<=agent.getMyID();a++){
	    s=s+"\nlevel"+a+"\n";
	    for(int v=0;v<variablesNr;v++){
		s=s+variables[v]+" "+levelIntersection[a][v]+
		    "<-"+mHCks[a][v]+"\n";
	    }
	}
	MySystem.out.println(s);
    }

    LabelsManager(int agentsNr, String[] variables, IOHashtable vIDs, PAgent agent){
	this.agent=agent;
	this.vIDs = vIDs;
	this.agentsNr = agentsNr;
	this.variables = variables;
	variablesNr = variables.length;
	mHCk  = new Range[variablesNr];
	basePID = new int[variablesNr];
	for(int v=0;v<variablesNr;v++) basePID[v]=0;
	levelIntersection = new ILCN[agentsNr][variablesNr];
	mHCks = new LCNHashtable[agentsNr][variablesNr];
	proposalID=new int[agentsNr][variablesNr];
	localInstantiations=new RHashtable[agentsNr];
	addedInstantiations=new RHashtable[agentsNr];
	for(int a=0;a<agentsNr;a++){
	    localInstantiations[a]=new RHashtable();
	    addedInstantiations[a]=new RHashtable();
	    for(int v=0;v<variablesNr;v++){
		mHCks[a][v]=new LCNHashtable();
		proposalID[a][v]=0;
	    }
	}
	consistentLevel=agent.getMyID()-1;
    }
    void addInternalRestrictingVariables(AssignmentRefs al){
	if(consistentLevel>=0)
	    for(int k=0;k<variablesNr;k++){
		ILCN inters=levelIntersection[consistentLevel][k];
		if((inters!=null)&&(inters.dep!=null))
		    al.addAll(inters.dep);
	    }
    }
    DVarHistory getLocalDV(AssignmentRef al){
	int[] hist=varHistory.geth(al.var);
	if(hist==null)MySystem.err.println("No last history for:"+al);
	int[] lhist=History.getHistory(hist,al.level);
	if(lhist==null)MySystem.err.println("No level history for:"+al);
	return new DVarHistory(localInstantiations[al.level].getR(al.var),
			       lhist);
    }
    DVarHistory getAddedDV(AssignmentRef al){
	return new DVarHistory(addedInstantiations[al.level].getR(al.var),
			       History.getHistory(addedHistory.geth(al.var),al.level));
    }
    DVarHistory getDV(String var, int level){
	RuntimeException eh=new RuntimeException("no hist at level:"+level+
						 " for var:"+var);
	RuntimeException er=new RuntimeException("no range at level"+level+
						 " for var:"+var);
	int[] hist=geth(var,level);
	if((hist==null)||(hist.length==0)){
	    if(hist!=null)
		MySystem.err.println("hist="+History.toString(hist));
	    printStructures();
	    throw eh;
	}
	if(hist[hist.length-2]!=level){
	    if(hist!=null)
		MySystem.err.println("hist="+History.toString(hist));
	    printStructures();
	    throw eh;
	}
	Range r=getInstantiation(var,level);
	if(r==null){
	    printStructures();
	    throw er;
	}
	return new DVarHistory(r,hist);
    }

    int[] geth(int id, String var){
	if(id>=0)return varHistory.geth(var);
	return addedHistory.geth(var);
    }
    int[] geth(String var){
	int[]hist=getLocalh(var);
	if(hist==null)hist=addedHistory.geth(var);
	return hist;
    }
    int[] geth(String var,int level) {
	int[]hist=geth(var);
	if(hist==null) return null;
	return History.getHistory(hist,level);
    }
    int[] getLocalh(String var){
	return varHistory.geth(var);
    }
    int[] getLocalh(String var, int level){
	int[] hist=getLocalh(var);
	if(hist==null) return null;
	return History.getHistory(hist,level);
    }
    AssignmentRef getLocalAR(String var, int level){
	int[]hist = getLocalh(var,level);
	if((hist==null)||(hist.length==0)) return null;
	return new AssignmentRef(var,hist);
    }
    DVarHistory getNewestDVinView(String var){
	int[]h=getLocalh(var);
	if((h==null)||(h.length==0)){
	    int vID=vIDs.geti(var);
	    return new DVarHistory(mHCk[vID],new int[0]);
	}
	int level=h[h.length-2];
	return new DVarHistory(localInstantiations[level].getR(var),h);
    }
    Range getInstantiation(String var, int level) {
	Range r=localInstantiations[level].getR(var);
	if(r==null)
	    r=addedInstantiations[level].getR(var);
	return r;
    }
    Range getLocalInstantiation(String var, int level){
	return localInstantiations[level].getR(var);
    }
    Range getLastLocalInstantiation(String var, int level){
	int[] hist=getLocalh(var,level);
	if((hist==null)||(hist.length==0)){
	    int vID=vIDs.geti(var);
	    return mHCk[vID];
	}
	int instLevel=hist[hist.length-2];
	return localInstantiations[instLevel].getR(var);
    }
//     void setPosition(int pos){
// 	position=pos;
// 	if(consistentLevel>pos)
// 	    consistentLevel=pos-1;
//     }
    //trimm histories and clear instantiations
    void cleanHistory(int changeLevel){
	ArrayList disabVar=varHistory.trimm(changeLevel);
	disabVar.addAll(addedHistory.trimm(changeLevel));
	for(int k=changeLevel+1;k<agentsNr;k++){
	    localInstantiations[k].clear();
	    addedInstantiations[k].clear();
	    for(int v=0;v<variablesNr;v++)
		mHCks[k][v].clean(changeLevel);
	}
    }
    //trimm histories and clear instantiations
    void cleanUpLevel(int changeLevel, String var,
		      int vID, IntObject minCG) {
	for(int k=changeLevel;k<agentsNr;k++){
	    if(vID>0){
		if(localInstantiations[k]!=null)
		    localInstantiations[k].remove(var);
	    }else{
		if(addedInstantiations[k]!=null)
		    addedInstantiations[k].remove(var);
	    }
	    for(int v=0;v<variablesNr;v++){
		if(mHCks[k][v].clean(var,changeLevel))
		    if(minCG.i>k) minCG.i=k;
		if((levelIntersection[k][v]!=null)&&
		   (levelIntersection[k][v].disabledBy(var,changeLevel))){
		    levelIntersection[k][v]=null;
		    if(minCG.i>k) minCG.i=k;
		}
	    }
	}
    }
    void setInitialDomains(Range[]initialDomains){
	for(int v=0;v<variablesNr;v++)
	    mHCk[v]=(Range)initialDomains[v].clone();
    }
    Range[] getRefBaseLevel(){
	return mHCk;
    }
    void setOrderChange(int changeLevel){
	for(int k=changeLevel+1; k<agent.getMyID(); k++){
	    for(int v=0;v<variablesNr;v++){
		int source=k-1;
		if(source>=0){
		    if(mHCks[source][v]!=null){
			mHCks[k][v]=  new LCNHashtable();
			if(levelIntersection[source][v]!=null)
			    levelIntersection[k][v]=
				(ILCN)levelIntersection[source][v].clone();
			else levelIntersection[k][v]=null;
		    }else{
			mHCks[k][v]=null;
			levelIntersection[k][v]=null;
		    }
		}else{
		    mHCks[k][v]=new LCNHashtable();
		    levelIntersection[k][v]=new ILCN(mHCk[v]);
		}
	    }
	}
	if(consistentLevel>=changeLevel) consistentLevel=agent.getMyID()-1;
    }
    void update0Level(){
	for(int k=0;k<agentsNr;k++)
	    for(int v=0;v<variablesNr;v++)
		levelIntersection[k][v]=new ILCN(mHCk[v]);
	consistentLevel=agent.getMyID()-1;
    }
    private AgentID getAgent(int k){
	return agent.getAgent(k);
    }
    private void addState(int level, LCNHashtable[] lState,
		  RHashtable[] insts, IntObject mincg) {
	int k=0;
	int myID=agent.getMyID();
	if(level<myID){
	    if(level>=0){
		for(;k<variablesNr;k++)
		    if(mHCks[level][k].addAll(lState[k]))
			if(mincg.i>level) mincg.i=level;
	    }else for(;k<variablesNr;k++){
		Range incRange=lState[k].getRange();
		if((incRange!=null)&&(mHCk[k].restricting(incRange))){
		    mHCk[k]=(Range)mHCk[k].clone();
		    mHCk[k].restrict(incRange);mincg.i=-1;
		}
	    }
	}
	for(k=0;(k<=level)&&(k<myID); k++){
	    if(insts[k]==null)continue;
	    Iterator it = (Iterator)insts[k].keys();
	    while(it.hasNext()){
		String var=(String)it.next();
		Range instOfVar=insts[k].getR(var);
		int vID=vIDs.geti(var);
 		putI(vID, var, instOfVar, k, mincg);
	    }
	}
    }
    private void putI(int vID, String var, Range instOfVar,
		      int crtLevel, IntObject mincg){
		if(vID>=0){
		    int[] vHist=getLocalh(var);
		    if(vHist==null)MySystem.err.println("no "+var+" "+vHist);
		    LCN instLCN=new LCN(var,instOfVar,crtLevel,
					History.getPID(vHist,crtLevel));
		    localInstantiations[crtLevel].put(var,instOfVar);
		    LCN o=mHCks[crtLevel][vID].put(getAgent(crtLevel),instLCN);
		    if((o==null)||(o.proposalID!=instLCN.proposalID))
			if(mincg.i>crtLevel) mincg.i=crtLevel;
		}else addedInstantiations[crtLevel].put(var,instOfVar);
    }
    private void puth(int id, String var, int[] hist){
	if(id>=0)varHistory.put(var,hist);
	else addedHistory.put(var,hist);
    }

    private void insureLinks(String var){
	if(!addedVars.contains(var)){
	    if(!linksToAdd.contains(var))
		linksToAdd.add(var);
	    addedVars.add(var);
	}
    }
    private Range getInters(int level, int vID){
	if(level>=0) return levelIntersection[level][vID].label;
	return mHCk[vID];
    }
    private boolean updateIntersectionsMarkChange(int k, boolean[] modifVars, BoolObject enlarged){
	if(k==-1){
	    for(int v=0;v<variablesNr;v++) modifVars[v]=true;
	    return true;
	}
	boolean result=false, notPrev=false;
	enlarged.b=false;
	for(int v=0;v<variablesNr;v++){
	    //MySystem.out.println(k+"v="+v+"oldInt "+levelIntersection[k][v]);
	    ILCN newIntersect=computeIntersect(k,v);
	    //MySystem.out.println("newInt "+newIntersect);
	    ILCN oldInters=levelIntersection[k][v];
	    //MySystem.out.println("oldInt "+oldInters);
	    modifVars[v]=false;
	    if(newIntersect.label.different(getInters(k-1,v)))
		notPrev=true;
	    if(oldInters==null){
		levelIntersection[k][v]=newIntersect;
		enlarged.b=modifVars[v]=result=true;
		MySystem.out.println("oldInt null");
	    }else{
		if(newIntersect.label.different(oldInters.label)){
		    if(newIntersect.label.restricting(oldInters.label))
			enlarged.b=true;
		    levelIntersection[k][v]=newIntersect;
		    modifVars[v]=true;
		    MySystem.out.println("oldInt true "+k+" v="+v);
		    result=true;
		}
	    }
	}
	return notPrev && result;
    }

    ILCN computeIntersect(int k, int var){
	ILCN intersect=mHCks[k][var].intersection();
	if(k==0){
	    if(intersect==null) return new ILCN(mHCk[var]);
	    intersect.label=Range.intersect(intersect.label,mHCk[var]);
	    return intersect;
	}
	if(intersect==null)
	    return (ILCN)levelIntersection[k-1][var].clone();
	intersect.addILCN(levelIntersection[k-1][var]);
	return intersect;
    }

//      private boolean MHC(Range[] doms, boolean[]mVars,
//  			Hashtable newLabeling, ArrayList newConstraints){
//  	return true;
//      }
//      CN getNogood(){return null;}
    private CN getCN(LCN lcn, int level, int varID){
	Context dep=new Context();
	if(lcn.dep!=null){
	    Iterator it=lcn.dep.iterator();
	    while(it.hasNext()){
		AssignmentRef ar=(AssignmentRef)it.next();
		dep.put(ar.var,getDV(ar.var,ar.level));
	    }
	}
	return new CN(variables[varID],lcn.label,dep,lcn.proposalID);
    }
//     private ArrayList getNewConstraint(int k){
// 	if(k<agent.getMyID()-1){
// 	    MySystem.out.println("No redundant constr at level:"+k);
// 	    return null;
// 	}
// 	ArrayList result=new ArrayList();
// 	agent.getSearchCoveredConstraint(result);
// 	return result;
//     }
    private boolean hc(int level,boolean[]mVars,
		       CNSet[]newLabeling,ArrayList newConstraints,
		       BoolObject emptyNogood){
	Range[] domains=new Range[variablesNr];
	for(int v=0;v<variablesNr;v++){
	    if(level>=0)
		domains[v]=new Range(levelIntersection[level][v].label);
	    else domains[v]=new Range(mHCk[v]);
	    if(domains[v].size()<=0){
		if(level<0)emptyNogood.b=true;
		return false;
	    }
	}
	boolean empty=false,useCL=false;
	Hashtable dep=new Hashtable();
	Hashtable vdep=new Hashtable();
	Hashtable cdep=new Hashtable();
	for(int k=0;k<variablesNr;k++){
	    if(mVars!=null)MySystem.out.print(""+mVars[k]);
	    else MySystem.out.print(mVars);
	    MySystem.out.println(variables[k]+"="+domains[k]);
	}
	MySystem.out.println("Consistency at level:"+level);
	if(!agent.mArtificial)
	    if(!HC.HC(domains,mVars,dep,null)) empty=true;
	if((!empty)&&(newConstraints!=null)&&(newConstraints.size()>0)){
	    if(mVars==null)mVars=new boolean[variablesNr];
	    for(int k=0;k<variablesNr;k++) mVars[k]=false;
	    if(!agent.mArtificial)
		if(!HC.HC(domains,mVars,vdep,newConstraints,cdep))
		    empty=true;
	    if(cdep.size()>0) useCL=true;
	}
	for(int k=0;k<variablesNr;k++)
	    MySystem.out.println(variables[k]+"="+domains[k]);
	boolean[]modifs=new boolean[variablesNr];
	if(level>=0)
	    getConsistencyResult(levelIntersection[level],level,domains,dep,
				 modifs,empty,emptyNogood,vdep,useCL);
	else{
	    if(empty){
		MySystem.out.println("emptyNogood at -1");
		emptyNogood.b=true;return false;
	    }
	    dep.putAll(vdep);//exact dependencies don't matter
	    Iterator it=(Iterator)dep.keys();
	    while(it.hasNext()){
		int v=((Integer)it.next()).intValue();
		mHCk[v]=domains[v];
		if(newLabeling[0]==null)newLabeling[0]=new CNSet();
		basePID[v]++;
		newLabeling[0].add(new CN(variables[v],mHCk[v],null,basePID[v]));
	    }
	    for(int k=0;k<variablesNr;k++)
		MySystem.out.println(variables[k]+"="+mHCk[k]);
	    return !empty;
	}
	if(emptyNogood.b)return false;
	for(int v=0;v<variablesNr;v++) {
	    if(modifs[v]){
		if(!empty)proposalID[level][v]++;
		LCN lcn=new LCN(levelIntersection[level][v],
				proposalID[level][v]);
		if(!empty) mHCks[level][v].put(agent.getMyself(),lcn);
		if(newLabeling[level+1]==null)newLabeling[level+1]=new CNSet();
		newLabeling[level+1].add(getCN(lcn,level,v));
	    }
	}
	return !empty;
    }
    private AssignmentRefs[] getConflicts(ILCN[] levels){
	AssignmentRefs[] cs=new AssignmentRefs[variablesNr];
	for(int k=0;k<variablesNr;k++){
	    cs[k]=(AssignmentRefs)levels[k].dep.clone();}
	return cs;
    }
    //pune in primul parametru dependintele si seteaza modifs si empty nogood
    private void getConsistencyResult(ILCN[] newlevels, int level,
				      Range[] ranges, Hashtable deps,
				      boolean[] modifs,boolean empty,
				      BoolObject emptyNogood, 
				      Hashtable vdeps, boolean useCL){
	boolean emptyVars=false;
	AssignmentRefs[] cs=null;
	if(!empty) cs=getConflicts(newlevels);
	for(int i=0;i<variablesNr;i++) modifs[i]=false;
	for(int k=0;k<variablesNr;k++){
	    if((empty)&&(ranges[k].size()>0)) continue;
	    ArrayList vars=(ArrayList)deps.get(new Integer(k));
	    if(vars==null){
		if(empty){
		    if(!useCL)MySystem.exit("no dep for "+variables[k]+ranges[k]);
		    emptyVars=true;
		    modifs[k]=true;
		    newlevels[k].label=ranges[k];
		    break;
		}else continue;
	    }
	    modifs[k]=true;
	    newlevels[k].label=ranges[k];
	    Iterator it=vars.iterator();
	    while(it.hasNext()) {
		int varID=((Integer)it.next()).intValue();
		if(varID==k) continue;
		if(empty) newlevels[k].dep.addAll(newlevels[varID].dep);
		else newlevels[k].dep.addAll(cs[varID]);
	    }
	    if((empty)&&(newlevels[k].dep.size()==0)&&!useCL){
		MySystem.out.println("emptyNogood at:"+variables[k]+
				     " level:"+level);
		emptyNogood.b=true;
	    }
	    if(empty) break;
	}
	if(useCL){
	    AssignmentRefs cl=agent.getConflictList();
	    for(int k=0;k<variablesNr;k++){
		if((empty)&&(ranges[k].size()>0)) continue;
		ArrayList vars=(ArrayList)vdeps.get(new Integer(k));
		if(vars==null){
		    if(empty){
			if(emptyVars)MySystem.exit("no vdep for "+variables[k]+ranges[k]);
			if(newlevels[k].dep.size()==0)
			    emptyNogood.b=true;
			break;
		    }else continue;
		}
		modifs[k]=true;
		newlevels[k].label=ranges[k];
		Iterator it=vars.iterator();
		while(it.hasNext()) {
		    int varID=((Integer)it.next()).intValue();
		    if(varID==k) continue;
		    if(empty) newlevels[k].dep.addAll(newlevels[varID].dep);
		    else newlevels[k].dep.addAll(cs[varID]);
		}
		newlevels[k].dep.addAll(cl);
		if((empty)&&(newlevels[k].dep.size()==0)){
		    MySystem.out.println("emptyNogood at:"+variables[k]+
					 " level:"+level);
		    emptyNogood.b=true;
		}
		if(empty) break;
	    }
	}
    }
    private void cleanIntersectionRenewHC(int sLevel, CNSet[] newLabels,
					  BoolObject emptyNogood) {
	int crtLevel,lastLevel=agent.getMyID()-1;
	//MySystem.out.println("in cleanIntersectionRenewHC");
	for(crtLevel=sLevel;crtLevel<lastLevel;crtLevel++){
	    boolean[] modifVars=new boolean[variablesNr];
	    BoolObject enlarged=new BoolObject(false);
	    boolean changed=updateIntersectionsMarkChange(crtLevel,modifVars,enlarged);
	    ArrayList newConstraints=agent.getNewConstraint(crtLevel);
	    if(changed||(newConstraints!=null)) {
		if(enlarged.b) modifVars=null;
		if(!hc(crtLevel,modifVars,newLabels,newConstraints,emptyNogood)){
		    consistentLevel=crtLevel-1;break;
		}
	    }
	}
	if(crtLevel==lastLevel) {
	    boolean[] modifVars=new boolean[variablesNr];
	    BoolObject enlarged=new BoolObject(false);
	    updateIntersectionsMarkChange(crtLevel,modifVars,enlarged);
	    if(enlarged.b) modifVars=null;
	    //MySystem.out.println("in cleanIntersectionRenewHC at last:"+crtLevel);
 	    ArrayList newConstraints=agent.getNewConstraint(crtLevel);
 	    if(!hc(crtLevel,modifVars,newLabels,newConstraints,emptyNogood))
 		consistentLevel=crtLevel-1;
	    else consistentLevel=lastLevel;
	}//else MySystem.out.println("in cleanIntersectionRenewHC no last");
    }
    Range[] getCurrentDomains(){
	if(consistentLevel==-1) return(Range[])mHCk.clone();
	Range[]domains=new Range[variablesNr];
	for(int k=0;k<variablesNr;k++){
	    domains[k]=levelIntersection[consistentLevel][k].label;
	}
	return domains;
    }

    // true - modif to instantiations
    // post - remain only the really new
    boolean setLevel(Hashtable posts[],Histories post, int level,
		     LCNHashtable[] lState, ConflictSet newAddedVars,
		     RHashtable[] insts, AssignmentRefs disabledAR,
		     CNSet[] newLabels, BoolObject emptyNogood,
		     BoolObject disabledCL) {
	MySystem.out.println("setLevel");
	printStructures();
	for(int cp=0;cp<agentsNr;cp++) posts[cp] = new Hashtable();
	linksToAdd=newAddedVars;
	IntObject cgLevel=new IntObject(agentsNr+1);
	boolean result=addVarHistory(posts,post,level,disabledAR,cgLevel);
	disabledCL.b=agent.cleanCL(disabledAR);
	MySystem.out.println("addVarH cgLevel.i="+cgLevel.i);
	addState(level,lState,insts,cgLevel);
	MySystem.out.println("adState cgLevel.i="+cgLevel.i);
	if(level==agent.getMyID()){
	    if(cgLevel.i>level-1)cgLevel.i=level-1;
	    int nCLevel=agent.getCLNogoodChangeLevel();
	    if(nCLevel<cgLevel.i)cgLevel.i=nCLevel;
	}
	MySystem.out.println("myID    cgLevel.i="+cgLevel.i);
	if(cgLevel.i<=consistentLevel+1)
	    cleanIntersectionRenewHC(cgLevel.i,newLabels, emptyNogood);
	printStructures();
	return result;
    }

    //sets a list of variables to clear at each level
    //sets the posts for the mHCKs
    //removes olds from post
    private boolean addVarHistory(Hashtable posts[],Histories post, int level,
				AssignmentRefs disabledAR, IntObject cgLevel){
	boolean solutionHistoryChanges=false;
	ArrayList toRemove=new ArrayList();
	Iterator it=(Iterator)post.keys();
	MySystem.out.println("addVarHistory:"+post);
	while(it.hasNext()) {
	    String var=(String) it.next();
	    int vID=vIDs.geti(var);
            int incHist[]=post.geth(var);
	    int oldHist[]=geth(vID,var);
	    if(oldHist==null) {
		puth(vID,var,incHist);
		MySystem.out.println("got:"+var+" "+History.toString(incHist));
		int changeLevel=incHist[0];
		disabledAR.addMin(var,changeLevel);
		//links are added only if in valid nogoods
		if(vID<0) insureLinks(var);
		else solutionHistoryChanges=true;
		//cleanUpLevel(changeLevel,var,vID,cgLevel);
		addPosts(posts,level,incHist,changeLevel,var);
	    }else{
		if(History.firstDisablesSecond(incHist,oldHist)){
		    puth(vID,var,incHist);
		    MySystem.out.println("got:"+var+" "+History.toString(incHist));
		    int changeLevel=History.changeLevel(oldHist,incHist);
		    disabledAR.addMin(var,changeLevel);
		    cleanUpLevel(changeLevel,var,vID,cgLevel);
		    addPosts(posts,level,incHist,changeLevel,var);
		    if(vID>=0) solutionHistoryChanges=true;
		}else toRemove.add(var);
	    }
	}
	for(int k=toRemove.size()-1;k>=0;k--)
	    post.remove((String)toRemove.get(k));
	return solutionHistoryChanges;
    }

    //should add the needed part of the incomingHist to outPosts
    //starting upward from changeLevel and skipping level
    private void addPosts(Hashtable[]posts,int level,int[] incHist,int changeLevel, String var){
	for(int k=changeLevel;k<agentsNr;k++){
	    if(k==level) continue;
	    int[] newHist=History.getHistory(incHist,k,agent.getMyID());
	    if(newHist==null) continue;
	    posts[k].put(var,newHist);
	}
    }
}
