/*
This program was designed and implemented by Silaghi, Marius-Calin.
The package sJavap was obtained from sJava (developed by Prof. Petitpierre)
The package platform was initially the semester project of Michel Galley in 2000
Copyright (c) 1998, 1999, 2000, 2001, 2002, 2004
*/
package hc;

import csp.*;
import aggreg.*;
import utils.*;
import java.util.*;

class MHCConstraint{
    Range[] mKBoxCrt;
    CRepresentation mat;
    int[] varIDs;
    int localVariablesNr;
    // Load the KBox from D
    MHCConstraint(Constraint c){
	varIDs=c.varIDs;
	mat=c.getRepresentation();
        localVariablesNr = c.getVariablesNr();
        mKBoxCrt = new Range[localVariablesNr];
    }
    void Update(Range[] KBox, Range[] D){
	for(int j=0; j<localVariablesNr; j++)
	    KBox[j]=(Range)D[varIDs[j]].clone();
    }

    // Shrink Bounds of mKBoxCrt, ret 0 if empty
    boolean ShrinkBounds(){
	for (int variab=0;variab<localVariablesNr; variab++){
	    if(!ShrinkLower(variab))
		return false;
	    if(!ShrinkUpper(variab))
		return false;
	}
	return true;
    }

    // Shrink the respective limits of variab in mKBoxCrt;
    boolean ShrinkLower(int crt){
	int crtvar, last=mKBoxCrt[crt].getU();
	for (crtvar=mKBoxCrt[crt].getL(); crtvar<=last; crtvar++){
	    if (!Shrinkable(crt, crtvar))
		break;
	}
	mKBoxCrt[crt].setL(crtvar);
	if (crtvar>last)
	    return false;
        //the update was previously here
	return true;
    }

    boolean ShrinkUpper(int crt){
	int crtvar, last=mKBoxCrt[crt].getL();
	for (crtvar=mKBoxCrt[crt].getU(); crtvar>=last; crtvar--){
	    if (!Shrinkable(crt, crtvar))
		break;
	}
	mKBoxCrt[crt].setU(crtvar);
	if (crtvar<last)
	    return false;
        //update was previously here
	return true;
    }

    // Is the value value of variable completly infeasible?
    boolean Shrinkable(int variable, int value){
	Range[] reducible=(Range[])mKBoxCrt.clone();
	reducible[variable]=new Range(value);
	return !mat.anyFeasible(reducible);
    }

    // returns 0 if empty constraint found
    // dependency is(Integer:ArrayList(Integers))
    boolean CheckBounds(ConstraintList Q, Range[] D, int indice, int caller, 
			Hashtable dependency, Hashtable cdependency){
	Update(mKBoxCrt, D);
        boolean emptied=false;
	if(!ShrinkBounds()){ //Empty relation
	    emptied=true;
	}
	boolean changed=false;
	for(int variab=0; variab<localVariablesNr; variab++)
	    if(Different(D,mKBoxCrt[variab], varIDs[variab])){
                if(emptied)
                  if(mKBoxCrt[variab].size()>0) continue;
		HC.InsertDependent(Q, varIDs[variab],
				   indice, caller);
		Integer var= new Integer(varIDs[variab]);
		ArrayList conf = (ArrayList)dependency.get(var);
		if(conf==null) conf = new ArrayList();
		for(int k=0; k<localVariablesNr; k++){
		    if(true ||(k!=variab)){
			Integer var_conflict=new Integer(varIDs[k]);
			if(!conf.contains(var_conflict))
			    conf.add(var_conflict);
			ArrayList al =(ArrayList)dependency.get(var_conflict);
			if(al!=null){
			    Iterator it=al.iterator();
			    while(it.hasNext()){
				Integer i=(Integer)it.next();
				if(i.equals(var))continue;
				if(!conf.contains(i))conf.add(i);
			    }
			}
		    }
		}
                dependency.put(var,conf);
		ArrayList cconf = (ArrayList)cdependency.get(var);
		if(cconf==null) cconf = new ArrayList();
		cconf.add(new Integer(caller));
		cdependency.put(var,cconf);
		changed= true;
	    }
	if(changed) UpdateDomain(D,emptied);
        if(emptied) return false;
	return true;
    }

    // returns 0 if empty constraint found
    boolean CheckBounds(ConstraintList Q, Range[] D,
			int indice, int caller){
	Update(mKBoxCrt, D);
	if(!ShrinkBounds()){ //Empty relation
	    return false;
	}
	boolean changed=false;
	for(int variab=0; variab<localVariablesNr; variab++)
	    if(Different(D,mKBoxCrt[variab], varIDs[variab])){
		HC.InsertDependent(Q, varIDs[variab],
				   indice, caller);
		changed= true;
	    }
	if(changed){
	    UpdateDomain(D);
	}
	return true;
    }

    boolean Conflicts(int[] Ind, int NrVariables){
	for(int i=0; i<NrVariables; i++)
	    for(int j=0; j<localVariablesNr; j++)
		if(Ind[i]==varIDs[j]) return true;
	return false;
    }

    boolean Conflicts(int Ind){
	for(int j=0; j<localVariablesNr; j++)
	    if(Ind==varIDs[j]) return true;
	return false;
    }

    boolean Different(Range[] dom, Range interval, int id) {
	return dom[id].different(interval);
    }
    void UpdateDomain(Range[] dom){
	for(int k=0; k<localVariablesNr; k++)
	    CopyRestraining(dom, mKBoxCrt[k], varIDs[k]);
    }
    void UpdateDomain(Range[] dom, boolean emptied){
	for(int k=0; k<localVariablesNr; k++){
	    if(emptied&&(mKBoxCrt[k].size()>0)) continue;
	    CopyRestraining(dom, mKBoxCrt[k], varIDs[k]);
	}
    }
    void CopyRestraining(Range[] dom, Range box,
			 int var){
	dom[var]=(Range)box.clone();
    }
}

public class HC{
    public static int constraintsNr;
    public static MHCConstraint[] constraints;
    public static int[] order;//pointer
    private static int pconstraintsNr;
    private static MHCConstraint[] pconstraints;
    private static int[] porder;//pointer

    public static void setConstraints(Constraint[] _constraints){
	constraints=new MHCConstraint[_constraints.length];
	for(int i=0;i<_constraints.length;i++)
	    constraints[i]=new MHCConstraint(_constraints[i]);
    }
    static MHCConstraint getConstraint(int c){
	return constraints[order[c]];
    }
    static public boolean mHC(Range[] crtDomain, int constraint){
	ConstraintList Q=new ConstraintList();
	int c;
        int k1=getConstraint(constraint).varIDs[0];
        int k2=getConstraint(constraint).varIDs[1];
        InsertDependent(Q, k1, constraint);
        InsertDependent(Q, k2, constraint);
	//Q.Insert(constraint);
	while((c=Q.ExtractConstraint())>=0){
	    if(!getConstraint(c).
	       CheckBounds(Q, crtDomain, constraint, c))
		return false;
	}
	return true;
    }

//    static public boolean HC(MHCHistory[] hcrtDomain){
//	ConstraintList Q=new ConstraintList();
//	Range[] crtDomain=new Range[hcrtDomain.length];
//	int c,k;
//	for(k=0; k<hcrtDomain.length; k++)
//	    crtDomain[k]=hcrtDomain[k].proposal;
//	for(k=0; k<constraintsNr; k++)
//	    Q.Insert(k);
//	while((c=Q.ExtractConstraint())>=0){
//	    if(!getConstraint(c).
//	       CheckBounds(Q, crtDomain, -1, c))
//		return false;
//	}
//	return true;
//  }
    // dependency should return the ArrayList of variable indices (Integer)
    // that have been used to modify the (Integer)index in key
    static public boolean HC(Range[] crtDomain, Hashtable dependency){
	return HC(crtDomain, dependency, new Hashtable());
    }
    static public boolean HC(Range[] crtDomain, Hashtable dependency,
			     Hashtable cdependency){
	ConstraintList Q=new ConstraintList();
	int c;
	for(int k=0; k<constraintsNr; k++)
	    Q.Insert(k);
	while((c=Q.ExtractConstraint())>=0){
	    if(!getConstraint(c).
	       CheckBounds(Q, crtDomain, -1, c, dependency, cdependency))
		return false;
	}
	return true;
    }

    static public boolean HC(Range[] crtDomain, Hashtable dependency, ArrayList queue){
	return HC(crtDomain, dependency, queue, new Hashtable());
    }
    //queue list of integers
    static public boolean HC(Range[] crtDomain, Hashtable dependency, ArrayList queue, Hashtable cdependency){
	ConstraintList Q=new ConstraintList();
	int c;
	Iterator it=queue.iterator();
	while(it.hasNext())
	    Q.Insert(((Integer)it.next()).intValue());
	while((c=Q.ExtractConstraint())>=0){
	    if(!getConstraint(c).
	       CheckBounds(Q, crtDomain, -1, c, dependency, cdependency))
		return false;
	}
	return true;
    }

    private static void addConstraint(Constraint c){
	constraintsNr++;
	int[]neworder=new int[constraintsNr];
	MHCConstraint[] newconstraints=new MHCConstraint[constraintsNr];
	for(int k=constraintsNr-2;k>=0;k--){
	    neworder[k]=order[k];
	    newconstraints[k]=constraints[k];
	}
	neworder[constraintsNr-1]=constraintsNr-1;
	newconstraints[constraintsNr-1]=new MHCConstraint(c);
	order=neworder;
	constraints=newconstraints;
    }
    private static void store(){
	pconstraintsNr=constraintsNr;
	porder=order;
	pconstraints=constraints;
    }
    private static void restore(){
	constraintsNr=pconstraintsNr;
	order=porder;
	constraints=pconstraints;
    }

    static public boolean HC(Range[] crtDomain, boolean[]modifVars, Hashtable dependency, ArrayList constraints) {
	return HC(crtDomain,modifVars,dependency,constraints,new Hashtable());
    }
    static public boolean HC(Range[] crtDomain, boolean[]modifVars, Hashtable dependency, ArrayList constraints, Hashtable cdependency) {
	ConstraintList Q=new ConstraintList();
	int c;
	if(constraints!=null){
	    store();
	    for(int k=constraints.size()-1;k>=0;k--)
		addConstraint((Constraint)constraints.get(k));
	}
	if(modifVars==null) {
	    for(int k=0; k<constraintsNr; k++)
		Q.Insert(k);
	}else{
	    for(int k=0;k<modifVars.length;k++)
		if(modifVars[k]) InsertDependent(Q,k,-1,-1);
	    if(constraints!=null) {
		for(int p=pconstraintsNr;p<constraintsNr;p++)
		    Q.Insert(p);
	    }
	}
	while((c=Q.ExtractConstraint())>=0){
	    if(!getConstraint(c).
	       CheckBounds(Q, crtDomain, -1, c, dependency, cdependency)){
		if(constraints!=null)restore();
		return false;
	    }
	}
	if(constraints!=null)restore();
	return true;
    }

    static public boolean HC(Range[] crtDomain){
	ConstraintList Q=new ConstraintList();
	int c;
	for(int k=0; k<constraintsNr; k++)
	    Q.Insert(k);
	while((c=Q.ExtractConstraint())>=0){
	    if(!getConstraint(c).
	       CheckBounds(Q, crtDomain, -1, c))
		return false;
	}
	return true;
    }

    // Insert in Q the constraints on variab, that follow instbase
    // and are different from caller
    static int InsertDependent(ConstraintList Q, int variab,
			int instbase){
	return InsertDependent(Q, variab, instbase, 0);
    }
    static int InsertDependent(ConstraintList Q, int variab,
			int instbase, int caller){
	int res=0;
	for(int k=instbase+1; k<constraintsNr; k++){
	    if (k==caller)
		continue;
	    if (getConstraint(k).
		Conflicts(variab))
		if(Q.Insert(k))
		    res++;
	}
	return res;
    }
}
