/*
This program was designed and implemented by Silaghi, Marius-Calin.
The package sJavap was obtained from sJava (developed by Prof. Petitpierre)
The package platform was initially the semester project of Michel Galley in 2000
Copyright (c) 1998, 1999, 2000, 2001, 2002, 2004
*/
package mads;
import java.util.*;
import csp.*;
import platform.*;
import parameters.*;
import utils.*;
import sjavap.*;

class MHCk implements MessageReceiver, Runnable{

 /**    active object support (needs implements Reply)   **/
 /*********************************************************/
 /**/ Synch_List select = new Synch_List(this);           //
 /**/ synchronized public void Synch_Reply()              //
 /**/   { notify();   }                                   //
 /**/ int now() {return Kernel.kern.currentTime;}         //
 /**/ int Synch_Wait() {                                  //
 /**/   Kernel.kern.Synch_Request(select);                //
 /**/   do{try {wait();} catch(InterruptedException e){}} //
 /**/   while(select.nbEntries!=0);                       //
 /**/   return select.Value; }                            //
 /*********************************************************/

    private MHCAgent MHCscheduler;
    private int variablesNr;
    private int level;
    private int myComputationNum;
    private MessageQueue queue;
    //Am I the level updating the instantiation?
    private boolean instantiation=false;
    private boolean fromLocal=false;//is the current message coming from local?
    //here we can be announced about modified histories
    //(String-var:int[]hist)
    private Histories outgoingPost=new Histories();
    private Histories post=new Histories();
    private Message_Command postPropagate=null;//propagate that can be sent to other level
    private Hashtable insts=new Hashtable();//incoming from scheduler
    private Hashtable myInst=new Hashtable();
    private Object postMonitor=new Object();
    // changed histories for disabling level decisions
    private Hashtable changed=new Hashtable();
    private DCost localCost=new DCost();
    //active methods
    final public static int _terminate=2;
    final public static int _getLocalMessage=3;
    final public static int _setInstantiation=4;

    boolean MHCmodif;
    //order
    private boolean orderModif=false;
    private Order order=new Order();
    private Order orderScheduler;//order of scheduler
    private int wHist=-1;
    private int wVol=0;
    private HeuristicContainer heurs;

    //states
    final static int Sreceiving=0;
    final static int SemptyReceiving=2;
    final static int Sstop=4;
    private int crtState;
    private Histories histories=new Histories();
    private MHCHistory[] levelState;
    private Message gotMessage=null;
    private String gotMsgSource=null;

    private AgentID myself=null;
    private AgentID[] defaultAgents=null;

    MHCk(MHCAgent scheduler, int level, int vars, int myComputationNum, MessageQueue queue, int agentsNr, AgentID myself, AgentID[] defaultAgents){
	variablesNr=vars;
	this.level=level;
	this.queue=queue;
	this.myComputationNum=myComputationNum;
	heurs=new HeuristicContainer(agentsNr,defaultAgents);
	levelState=new MHCHistory[variablesNr];
	MHCscheduler=scheduler;
	set(myself,defaultAgents);
    }

    MHCk(MHCAgent scheduler, int level, int vars, boolean instant, int myComputationNum, MessageQueue queue, int agentsNr, AgentID myself, AgentID[] defaultAgents){
	instantiation=true;
	variablesNr=vars;
	this.level=level;
	this.queue=queue;
	this.myComputationNum=myComputationNum;
	heurs=new HeuristicContainer(agentsNr,defaultAgents);
	levelState=new MHCHistory[variablesNr];
	MHCscheduler=scheduler;
	set(myself,defaultAgents);
    }
    private int getMyPosition(Order order){
	int k;
	if((order==null)||(order.sequence==null)){
	    MySystem.err.println("got:"+order);
	    MySystem.exit(1);
	}
	if((myself==null)){
	    MySystem.err.println("myself:"+myself);
	    MySystem.exit(1);
	}
	Hashtable h=new Hashtable();
	for(k=0;k<order.sequence.length;k++){
	    if(myself.equals(order.sequence[k])) return k;
	    h.put(order.sequence[k],myself);
	}
	if((defaultAgents==null)){
	    MySystem.err.println("defaultAgents:"+defaultAgents);
	    MySystem.exit(1);
	}
	//MySystem.err.println("defaultAgents:"+defaultAgents[0]);
	for(int j=0;j<defaultAgents.length;j++){
	    if(h.get(defaultAgents[j])==null){
		if(myself.equals(defaultAgents[j])) return k;
		else k++;
	    }
	}
	return -1;
    }

    void set(AgentID myself, AgentID[] defaultAgents){
	this.myself=myself;
	this.defaultAgents=defaultAgents;
    }

    public int getLevel(){
	return level;
    }

    synchronized public void setInstantiation(Order order, boolean b){
	fromLocal=true;
	if(getOrder(order))
	    if(instantiation!=b){
		MySystem.err.println("incOrder="+order+b+instantiation);
		MySystem.exit(1);
	    }
	fromLocal=false;
    }

    private void checkPostPropagate(){
	if(postPropagate!=null){
	    int level=((Propagate)postPropagate.getArguments()).level;
	    if(level<=this.level) MySystem.exit(1);
	    MHCk nextC=MHCscheduler.getMHCk(level);
	    select.accept(0,_terminate);
	    select.call(1,nextC,nextC._getMessage);
	    if(Synch_Wait()==1){
		nextC.getMessage(postPropagate);
		postPropagate=null;
	    }
	}
    }

    synchronized public void run() {
	try{
	    rrun();
	}catch(Exception e){
	    e.printStackTrace();
	    MySystem.exit(1);
	}
    }

    synchronized public void rrun() {
	select.thread=Thread.currentThread();
	crtState=SemptyReceiving;
	for(;;){
	    //if(instantiation) MySystem.err.println("MHCK level "+level+" in state:"+crtState);
	    if(instantiation)printLevelState();
	    switch(crtState){
	    case Sstop:
		return;

	    case SemptyReceiving:
		MHCmodif=false;
                select.accept(0,_terminate);
		select.accept(1,_getMessage,1+level);
		select.accept(2,_getLocalMessage);
		select.accept(3,_setInstantiation);
		int sWait = Synch_Wait();
		if(sWait==1){
                    MySystem.err.println("gotMessage="+gotMessage);
		    getMessage(); //get external Message
		    checkPostPropagate();
		}
		if(instantiation)
		    MySystem.err.println("MHCK level "+level+" in state:"+crtState+" after choice:"+sWait);
		if(sWait==0){
                    crtState=Sstop;
                    break;
                  }
		if(level>=0) //no post for base
		    readPostToChanged();
		//Only if remote info has arrived
		if(MHCmodif&&(crtState!=Sstop)){//&&(sWait!=2)){
		    MySystem.err.println("MHCK level "+level+" goes to modified state:"+crtState);
		    crtState=Sreceiving;
// 		    MHCscheduler.interruptRequest();
		}
		break;
	    case Sreceiving:
                select.accept(0,_terminate);
		select.accept(1,_getMessage,1+level);
		select.call(2,MHCscheduler,MHCscheduler._MHCk,1+(1+level));
		select.accept(3,_getLocalMessage);
		select.accept(4,_setInstantiation);
		int choice=Synch_Wait();
		if(choice==1){
		    getMessage(); //get external Message
		    checkPostPropagate();
		}
		//if(instantiation) MySystem.err.println("MHCK level "+level+" in state:"+crtState+" after choice:"+choice);
                if(choice==0){
                  crtState=Sstop; break;
                }
		if(level>=0) //no post for base
		    readPostToChanged();
		if(choice==2){
		    prepareLevels();
// 		    MHCscheduler.clearInterrupt();
		    ((DCost)Utils.agentCost).updateCost(localCost);
		    MHCscheduler.heurs.merge(heurs,level);
		    //if(instantiation) MHCscheduler.mHCk(levelState, outgoingPost,insts);else
		    MHCscheduler.mHCk(levelState,outgoingPost.getHashtable(),insts,level,order,wHist,wVol);
		    outgoingPost.clear(); //already processed
// 		    if(okMessage!=null){
// 			MHCscheduler.getIncomingOK(okMessage);
// 			okMessage=null;
// 		    }
		    crtState=SemptyReceiving;
		    MySystem.err.println("MHCK level "+level+" goes to state:"+crtState);
		}
		break;
	    }
	}
    }

    public synchronized void terminate(){
	crtState=Sstop;
        Synch_Reply();
    }

    public synchronized void getLocalMessage(Propagate m){
	MySystem.err.println("Agent enters getLocalMessage:"+m);
	fromLocal=true;
	if(level>=0) //no post for base
	    readPostToChanged();
	//if(instantiation) clearLevel();
	if(getOrder(m.order))
	    getMessage(m,myself);
	fromLocal=false;
        Synch_Reply();
    }
    public synchronized void getMessage(Message message){
	gotMessage=message;
	gotMsgSource=Utils.getStack();
	if(message.getComputationNum()!=myComputationNum){
	    gotMessage=null;
	    MySystem.err.println("mHCk:"+level+":!!my myComputationNum="+myComputationNum);
	    MySystem.err.println("mHCk:"+level+":!!my getComputationNum()="+message.getComputationNum());
	    MySystem.exit(1);//why-debug
	}
        Synch_Reply();
    }
    private void getMessage(){
	Message message=gotMessage;
	if(message==null){
	    MySystem.exit(1);//why-debug
	    return;
	}
	if(level>=0) //no post for base
	    readPostToChanged();
	Object argument=((Message_Command)message).getArguments();
	MySystem.err.println("MHCk:"+level+":Got message:"+argument);
	if(instantiation&&(argument instanceof DNogood)){
	    getNogood((DNogood)argument); //the order taken in scheduler and arrived by post
	    return;
	}
	if(argument instanceof Propagate){
	    Propagate mMHC=(Propagate)argument;
	    if(!getOrder(mMHC.order)) return;
	    postPropagate = mMHC.removeNext((Message_Command)message);
	    localCost.incrementMessages(mMHC.cost);
	    getMessage(mMHC,message.getSenderID());
	    return;
	}
	if(argument instanceof DOrder){
	    DOrder incOrder=(DOrder)argument;
	    if(!message.getReceiverID().equals(message.getSenderID()))
		localCost.incrementMessages(incOrder.cost);
	    getOrder(incOrder.order);
	    return;
	}
	DView mMHC=(DView)argument;
	if(!getOrder(mMHC.order)) return;
	localCost.incrementMessages(mMHC.cost);
	getContext(mMHC.view,true);
	disableChanged();
    }
    private boolean getOrder(Order incOrder){
	MySystem.err.println("incOrder="+incOrder);
	MySystem.err.println("oldOrder="+order);
	if(incOrder.getInterestLevel()>level){
	    System.err.println("inappropriate order");
	    System.err.println("incOrder="+incOrder);
	    System.err.println("oldOrder="+order);
	    System.err.println("message was="+gotMessage);
	    System.err.println("level was="+level);
	    MHCSelector sele=new MHCSelector(level);
	    System.err.println("selector returns:"+sele.accepts((Message_Command)gotMessage));
	    System.err.println("message source="+gotMsgSource);
	    MySystem.exit(1);
	}
	if(History.firstDisablesSecond(order.history,incOrder.history)){
	    MySystem.err.println("incoming old");
	    return false;
	}
	if(History.firstDisablesSecond(incOrder.history,order.history)){
	    MySystem.err.println("incoming new:delete all");
	    heurs.setOrder(incOrder);
	    resetOrderDependentHistory(incOrder.changeLevel(order));
	    order=incOrder;
	    wHist=-1;
	    wVol=-1;
	    instantiation=(level==getMyPosition(order));
	    if(!fromLocal){
		MHCmodif=true;orderModif=true;
		MySystem.err.println("modif in getOrder. Level="+level);
	    }
	}
	return true;
    }

    private void clearInsts(Hashtable insts ,ArrayList vars){
	Iterator it=vars.iterator();
	while(it.hasNext())
	    insts.remove(it.next());
    }

    private void resetOrderDependentHistory(int changeLevel){
	ArrayList disabVar=histories.trimm(changeLevel);
	levelState=new MHCHistory[levelState.length];
	clearInsts(myInst,disabVar);
	clearInsts(insts,disabVar);
	outgoingPost.trimm(changeLevel);
	//changed can remain unchanged since any disabled is disabled
    }
    private void getNogood(DNogood dn){
	if(validNogood(dn.view)){
	    MHCmodif=true;
	    localCost.incrementMessages(dn.cost);
	    MySystem.err.println("MHCk:Valid nogood:modif");
	    registerNogood(dn);
	}else
	    MySystem.err.println("MHCk:Invalid nogood");
    }
    private void getMessage(Propagate m,AgentID sender){
	int i,j,k;
	if(m.addedLinks.size()>0)
	    getContext(m.addedLinks,true);

	mergeHistory(m.varHistories);
	heurs.updateHeurHistories(histories);
	if(heurs.put(sender,m.heuristic,level)&&!fromLocal){
	    MySystem.err.println("MHCmodif:heuristic:Level="+level);
	    MHCmodif=true;
	}
	if(heurs.put(sender,m.varHistories,level)&&!fromLocal){
	    MySystem.err.println("MHCmodif:varHistories:Level="+level);
	    MHCmodif=true;
	}
	Hashtable what= new Hashtable();
	//heurs.updateParamHistories(histories,what,changed);
	disableChanged();
	for(k=0; k<m.varHistories.size(); k++){
	    Label prop=
		m.varHistories.getL(k);
	    updateKnowledge(new MHCHistory(prop));
	}
    }
    //if some element of the Context is old, the other new ones
    //are taken only if takeDisabled
    private boolean getContext(Context m, boolean takeDisabled){
	boolean disabled;
	disabled=false;
	String variable;
	int[] incomingHist;
	int[] localHist;
	if(heurs.getContext(m)&&!fromLocal){
	    MHCmodif=true;
	    MySystem.err.println("MHCmodif:getContext:Level="+level);
	}
	Hashtable tmpHist=new Hashtable();
	Hashtable tmpPost=new Hashtable();
	Hashtable tmpInsts=new Hashtable();
	Iterator it=(Iterator)m.keys();
	while(it.hasNext()){
	    variable=(String)it.next();
	    Range incomingRange=m.getR(variable);
	    incomingHist=m.getH(variable).history;
	    //if(incomingHist==null)   MySystem.exit(1);
	    if((incomingHist==null)||(incomingHist.length==0))
		continue;
	    localHist=histories.geth(variable);
	    if(localHist!=null) {
		MySystem.err.println("Got history:"+History.toString(incomingHist)+" for "+variable+" old was "+History.toString(localHist));
		if(History.firstDisablesSecond(incomingHist,localHist)){
		    MySystem.err.println("Got newer history:"+History.toString(incomingHist)+" for "+variable);
		    changed.put(variable,incomingHist);
		    histories.put(variable,incomingHist);
		    if((!instantiation)||
		       (incomingHist[incomingHist.length-2]<level)){
			if(!fromLocal){
			    MHCmodif=true;
			    MySystem.err.println("MHCmodif:getContext:inst:Level="+level);
			}
			if(incomingRange!=null)
			    insts.put(variable,incomingRange);
			else insts.remove(variable);
			outgoingPost.put(variable,incomingHist);
		    }else{
			if(incomingRange!=null)
			    myInst.put(variable,incomingRange);
			if(History.baseDisablesSecond(incomingHist,localHist))
			    insts.remove(variable);
		    }
		}else{
		    if(History.firstDisablesSecond(localHist,incomingHist)){
			MySystem.err.println(variable+" "+
					     History.toString(incomingHist)+
					     " disabled due to "+
					     History.toString(localHist));
			disabled=true;
			//labeling.remove(k);break;//must continue
		    }else{//perhaps there was no instantiation
			//but myInst was surely there
			if(incomingRange!=null)
			    if((!instantiation)||
			       (incomingHist[incomingHist.length-2]<level))
				insts.put(variable,incomingRange);
		    }
		}
	    }else{
		MySystem.err.println("Got new history:"+History.toString(incomingHist)+" for "+variable);
		tmpHist.put(variable,incomingHist);
		if((!instantiation)||
		   (incomingHist[incomingHist.length-2]<level)){
		    tmpPost.put(variable,incomingHist);
		    if(incomingRange!=null)
			tmpInsts.put(variable,incomingRange);
		}else
		    if(incomingRange!=null)
			myInst.put(variable,incomingRange);
	    }
	}
	if(takeDisabled||!disabled){
	    histories.putAll(tmpHist);
	    changed.putAll(tmpHist);
	    insts.putAll(tmpInsts);
	    if(tmpPost.size()>0)
		if(!fromLocal){
		    MHCmodif=true;
		    MySystem.err.println("MHCmodif:getContext:take-dis:Level="+level);
		}
	    outgoingPost.putAll(tmpPost);
	}
	return disabled;
    }

    private void disableChanged(){
	if(changed.size()>0){
	    for(int k=0; k<levelState.length; k++){
		if((levelState[k]!=null)&&levelState[k].disabledBy(changed)){
		    if(!(instantiation&&levelState[k].disabledHistoryBy(changed))&&!fromLocal){
			MySystem.err.println("MHCmodif:disable:Level="+level);
			MHCmodif=true;
		    }
		    if((instantiation)&&
		       (!levelState[k].disabledHistoryBy(changed))){
			levelState[k].proposal=
			    (Range)myInst.get(levelState[k].variable);
			levelState[k].dependencies=new ConflictSet();
		    }else{
			levelState[k]=null;
		    }
		}
	    }
	    changed.clear();
	}
    }

    //merge history, erase old proposals, return new history
    void mergeHistory(Labeling labeling){
	boolean disabled;
	Object removed;
	for(int k=labeling.size()-1; k>=0; k--){
	    Label prop = labeling.getL(k);
	    if(disabled=getContext(prop.context,false)){//do not take disabled
		removed=labeling.remove(k);
		MySystem.err.println("Removing incoming label:"+removed+":at level:"+level);
	    }
	}
    }
    //-- if new range for a variable, then
    //--reset all the dependent instantiations
    //-- if new history for a variable, then
    //--update all the dependent instantiations
    //if old history then discard proposal
    void updateKnowledge(MHCHistory prop){
	int idprop=MHCscheduler.getID(prop.variable);
        if(idprop<0){
	    MySystem.err.println("Error bad ID at:"+prop);
	    MySystem.exit(1); //old message?
	}
	if((levelState[idprop]==null)){
	    levelState[idprop]=(MHCHistory)prop.clone();
	    if(!fromLocal){
		MySystem.err.println("MHCmodif:updateKnowledge");
		MHCmodif=true;
	    }
	}else{
	    //if(!MHCHistory.disablingHistoryOf(prop, levelState[idprop]))
	    boolean differ=prop.proposal.different(levelState[idprop].proposal);
	    MHCHistory sol=MHCHistory.intersect(prop,
						levelState[idprop]);
	    if(sol!=null) {
		if(sol!=prop){ //combination
		    MySystem.err.println("MHCmodif:updateKnowledge:sol!=prop");
		    MHCmodif=true;
		}
		levelState[idprop]=sol;
		if(!fromLocal){
		    MySystem.err.println("MHCmodif:updateKnowledge:fromL");
		    MHCmodif=true;
		}
	    }else{ //unchanged
		if(fromLocal&&differ){
		    MySystem.err.println("MHCmodif:updateKnowledge:differ");
		    MHCmodif=true;
		}
	    }
	}
    }
    void prepareLevels(){
	for(int k=0; k<levelState.length; k++){
	    if(levelState[k]==null){
		String var=MHCscheduler.variables[k];
		Range inst=(Range)insts.get(var);
		if(inst!=null)
		    levelState[k]=new MHCHistory(0,var,inst,null);
	    }
	}
    }

    private void printLevelState(){
	for(int i=0;i<levelState.length;i++)
	    if(levelState[i]==null)MySystem.err.println("null");
	    else MySystem.err.println(levelState[i]);
    }

    //tests if a nogood is denying my current instantiation
    private boolean validNogood(Context view){
	if(emptyLevel(levelState)){
	    MySystem.err.println("Nogood invalid since empty levelState");
	    printLevelState();
	    return false;
	}
	Iterator it= (Iterator)view.keys();
	while(it.hasNext()){
	    String var= (String)it.next();
	    int varID=MHCscheduler.getID(var);
	    DVarHistory valuation=view.getDV(var);
	    //It is sufficient that one view is outdated
	    //for invalidating the nogood
	    if(outdatedView(var,varID,valuation)){
		MySystem.err.println("Nogood invalid since outdated view");
		return false;
	    }
	}
	return true;
    }

    // test whether an element of a nogood is outdated
    private boolean outdatedView(String var, int varID, DVarHistory valuation){
	int[] old=histories.geth(var);
	DVarHistory oldVH =new DVarHistory((Range)null,old);
	if((old!=null)&&valuation.olderThan(oldVH)){
	    MySystem.err.println("MHCk:old hist:Nogood outdated at var:"+var+" I had:"+oldVH);
	    return true;
	}
// 	if(varID<0)
// 	    return false;
// 	MHCHistory ls=levelState[varID];
// 	if(ls==null){
// 	    MySystem.err.println("MHCk:ls=null:Nogood outdated at var:"+var);
// 	    return true;
// 	}
	return false;
    }

    //saves needed variables in linksToAdd respectively to conflictingVariables
    private void registerNogood(DNogood nogood){
	if(getContext(nogood.view,false))
	    MySystem.exit(1);//invalid nogood
	disableChanged();
	int vid=getLocalID(nogood.view);
	if(vid<0){
	    vid=0;
	    MySystem.err.println("MHCK inapropriate incoming nogood");
	    MySystem.exit(1);
	}
	if(levelState[vid]==null) {
	    String var=MHCscheduler.getVariable(vid);
	    MySystem.err.println("MHCK inst:"+vid+" was empty:will not register nogood at:"+var);
	    levelState[vid]=new MHCHistory(level, var,
					   new Range(2,1), null);
	    //MySystem.exit(1);//was valid
	    //return; //we have just disabled it somehow
	}
	levelState[vid].proposal.setU(levelState[vid].proposal.getL()-1);
	//ConflictSet cs=new ConflictSet(nogood.view.keySet());
	//levelState[vid].dependencies.addAll(cs);
	String var=levelState[vid].variable;
	Iterator it=(Iterator)nogood.view.keys();
	while(it.hasNext()) {
	    String dvar=(String)it.next();
	    if(dvar.equals(var)) continue;
	    int[] dhist=nogood.view.geth(dvar);
	    //if(dhist[dhist.length-2]<level)//always inst
	    levelState[vid].dependencies.add(dvar);
	}
    }

    private void clearLevel(){
	for(int k=0;k<levelState.length;k++)
	    levelState[k]=null;
    }

    private int getLocalID(Context nogood){
	Iterator it=(Iterator)nogood.keys();
	while(it.hasNext()){
	    String var=(String)it.next();
	    int[] vHist=nogood.geth(var);
	    if(vHist[vHist.length-2]==level)
		return MHCscheduler.getID(var);
	}
	return -1;
    }

    boolean emptyLevel(MHCHistory[] doms){
	for(int k=0;k<doms.length;k++){
	    if(doms[k]==null)// return true;
		continue;
	    if(doms[k].proposal.size()<=0)
		return true;
	}
	return false;
    }

    void setPost(Hashtable incomingPost, Order order){
	synchronized(postMonitor){
	    MySystem.err.println("set post:"+level);
	    orderScheduler=order;
	    Iterator it=(Iterator)incomingPost.keys();
	    while(it.hasNext()){
		String var=(String)it.next();
		int[] postedHist=(int[])incomingPost.get(var);
		if((postedHist!=null)&&
		   (postedHist.length>1)&&
		   (postedHist[postedHist.length-2]<=level)){
		    int[] oldHistory=post.geth(var);
		    if((oldHistory==null)||
		       MHCHistory.firstDisablesSecond(postedHist,oldHistory))
			post.put(var,postedHist);
		}else{
		    MySystem.err.println(History.toString(postedHist));
		    MySystem.exit(1);
		}
	    }
	}
    }

    void clearPostTellOrder(Order order,int changeLevel){
	synchronized(postMonitor) {
	    MySystem.err.println("clear tell post:"+level);
	    post.trimm(changeLevel);
	    orderScheduler=order;
	}
    }

    //read and clear the post
    void readPostToChanged(){
	synchronized(postMonitor){
	    if(orderScheduler!=null){
		if(order.IDisableParam(orderScheduler)){
		    int changeLevel=orderScheduler.changeLevel(order);
		    MySystem.err.println("Read post:"+level);
		    post.trimm(changeLevel);
		}
		getOrder(orderScheduler);
	    }
	    heurs.updateHeurHistories(post);
	    Iterator it=post.keys();
	    while(it.hasNext()){
		String var=(String)it.next();
		int[] incomingHist=post.geth(var);
		int[] oldHistory=histories.geth(var);
		MySystem.err.println("Read post:l:"+level+":v="+var+":"+History.toString(incomingHist)+":had:"+History.toString(oldHistory));
		//if i am dependent on older
		if((oldHistory!=null)&&
		   MHCHistory.firstDisablesSecond(incomingHist,
						  oldHistory)){
		    changed.put(var,incomingHist);
		    histories.put(var,incomingHist);
		    //avoid posting the old value....
		    if(outgoingPost.geth(var)!=null)
			outgoingPost.remove(var);
		    insts.remove(var);
		}
		if(oldHistory==null){
		    // could used separate "unusedHistory"
		    histories.put(var,incomingHist);
		    changed.put(var,incomingHist);
		}
	    }
	    post.clear();
            MySystem.err.println("will DisableChanges");
	    disableChanged();
	}
    }
}
