/*
This program was designed and implemented by Silaghi, Marius-Calin.
The package sJavap was obtained from sJava (developed by Prof. Petitpierre)
The package platform was initially the semester project of Michel Galley in 2000
Copyright (c) 1998, 1999, 2000, 2001, 2002, 2004
*/
package parameters;
import utils.*;

public class History{
    public int[] history;

    public History(int[] hist){
	history=hist;
    }
    public History(DVarHistory dv){
	if(dv!=null)
	    history=dv.history;
    }
    public static int getLevel(int[]hist){
	return hist[hist.length-2];
    }
    public static int getPID(int[] hist, int agentID){
	for(int k=0;k<hist.length;k++,k++){
	    if(hist[k]==agentID)
		return hist[k+1];
	}
	return -1;
    }
    public static int[] buildHistory(int[] base, int myID, int id){
	int[] result= new int[base.length+2];
	int k;
	for(k=0;k<base.length;k++){
	    result[k]=base[k];
	}
	result[k++]=myID;
	result[k]=id;
	return result;
    }
    public static boolean firstInvalidatesSecond(int[] first, int[] second){
	return firstDisablesSecond(first,second)&&
	    !isIdentical(second,getHistory(first,getLevel(second)));
    }
    public static boolean firstDisablesSecond(int[] first,int[] nou){
	return !olderOrEqualThan(first,nou,-1);
    }
    public static boolean baseDisablesSecond(int[] first,int[] nou){
	return !olderOrEqualThan(first,nou,first.length-2);
    }
    // true if first older than second
    public static boolean olderThan(int[] first, int[] second){
	return !olderOrEqualThan(second,first,-1);
    }


    // true if first older or equal than nou
    public static boolean olderOrEqualThan(int[] first, int[] nou, int flength){
	int cmpsize; // the common length
	int hsize=nou.length>>1;
	int mysize=first.length>>1;
	if(flength>=0)
	    mysize=flength>>1;
	cmpsize=(mysize<hsize)?mysize:hsize;
	int kk;
	for(int k=0;k<cmpsize;k++){
	    kk=k<<1;
	    if(nou[kk]!=first[kk]){
		// if incoming by earlier constraint
		return first[kk]>nou[kk];
	    }else
		if(nou[kk+1]!=first[kk+1]){
		    // if incoming by earlier announce
		    return first[kk+1]<nou[kk+1];
		}
	}
	// if the common part of the history is identical
	// then this is older only if its history is shorter or equal
	return mysize<=hsize;
    }

    // true if history is consequence of parameter
    // ended by the constraint and order
    public static boolean crtConsequence(int[] history, int[] base ,int constraint, int order){
	boolean result=false;
	int cmpsize; // the common length
	int k=0;
	int mysize=history.length>>1;
	int hsize=base.length>>1;
	if(mysize!=hsize+1)
	    return false;
	int kk;
	for(;k<hsize;k++){
	    kk=k<<1;
	    //System.out.println("Checking history:"+k);
	    if(base[kk]!=history[kk])
		return false;
	    if(base[kk+1]!=history[kk+1])
		return false;
	}
	kk=k<<1;
	// if the common part of the history is identical
	if(history[kk]!=constraint)
	    return false;
	if(history[kk+1]!=order)
	    return false;

	return true;
    }
    
    //get subhistory including at most agent "level"
    public static int[] getHistory(int[] history, int level){
	int len=0;
	for (;len<history.length;len++,len++)
	    if(history[len]>level) break;
	//if(len==0) return null;//this may be used without tests
	int[] result=new int[len];
	for(int i=0;i<len;i++) result[i]=history[i];
	return result;
    }
    
    //get subhistory including at most agent "level"
    public static int[] getHistory(int[] history, int level, int maxID){
	int len;
	if(level<maxID) {
	    for(len=0;len<history.length;len++,len++)
		if(history[len]>level) break;
	}else len=history.length;
	if(len==0) return null;
	int[] result=new int[len];
	for(int i=0;i<len;i++) result[i]=history[i];
	return result;
    }

    //works for non empty parameters
    public static int changeRLevel(int[]first, int[] dVarHistory){
	if(firstDisablesSecond(first,dVarHistory))
	    return History.changeLevel(dVarHistory,first);
	else return History.changeLevel(first,dVarHistory);	
    }

    //called when first older than second in order to
    //find the first different agent
    public static int changeLevel(int[]first, int[] dVarHistory){
	int len=first.length;
	if(len>dVarHistory.length)
	    len=dVarHistory.length;
	for(int k=0;k<len;k++){
	    //if different agents here
	    if(first[k]!=dVarHistory[k])
		if(first[k]>dVarHistory[k])
		    return dVarHistory[k];
		else
		    return first[k];
	    k++;
	    //if different instantiation of the same agent
	    if(first[k]!=dVarHistory[k])
		return first[k-1];
	}
	if(first.length==dVarHistory.length)
	    MySystem.exit(1);//first older->common diff or first short
	if(first.length>dVarHistory.length){
	    MySystem.err.println("first="+toString(first));
	    MySystem.err.println("second="+toString(dVarHistory));
	    MySystem.exit(1);//first older->common diff or first short
	    return first[dVarHistory.length];
	}else{
	    return dVarHistory[first.length];
	}
    }


    // true if first is consequence of second, following level
    // this should be identical with second for all agents<=level
    // dVarHistory should be hist of level level
    public static boolean consequence(int[]first, int[] dVarHistory,int level){
	int newthislength=first.length;
	for(int k=0;k<first.length;k++,k++){
	    if(first[k]<=level) continue;
	    newthislength=k; break;
	}
	return isIdentical(first,dVarHistory,newthislength);
    }
    public static boolean consequence(int[] cq,int[] premise){
	if((cq.length>=premise.length)&&
	   (isIdentical(cq,premise,premise.length)))
	    return true;
	return false;
    }
    //are the histories identical? first being not null
    public static boolean isIdentical(int[] first,int[] dVarHistory){
	return isIdentical(first,dVarHistory,-1);
    }
    public static boolean isIdentical(int[] first,int[] dVarHistory, int flength){
	if(flength<0)
	    flength=first.length;
	if(flength!=dVarHistory.length)
	    return false;
	for(int k=0;k<flength;k++)
	    if(first[k]!=dVarHistory[k])
		return false;
	return true;
    }

    public static String toString(int[] history){
	String s;
	s="|";
	if(history==null){
	    s="";
	    return s;
	}
	if(history.length==0) s="||";
	for(int k=0;k<history.length;k++,k++)
	    s=s+history[k]+":"+history[k+1]+"|";
	return s;
    }
    public String toString(){
	String s;
	s=":h:"+toString(history);
	return s;
    }
}
