/*
The package platform is a semester project by Galley, Michel,
that Silaghi, Marius-Calin adviced in Spring 2000.
Several modifications were done subsequently by Silaghi, Marius-Calin.
Copyright (c) 2000, 2001, 2002, 2004
*/
/*
 * @(#)AgentGUI.java  1.0 00/02/04
 *
 * Intelligent Agents Collaborating in Distributed Problem Solving
 *
 * Supervisors:   Marius-Calin Silaghi, Djamila Sam-Haroud 
 * Author:        Michel Galley
 */

package platform;

import csp.*;
import java.io.*;
import javax.swing.*;
import javax.swing.filechooser.*;
import javax.swing.plaf.basic.*;
import java.awt.*;
import java.awt.event.*;

/**
 * This class defines a new swing component extending JFrame. It is a view
 * for the Agent class, enabling the user to manually subscribe, unsubscribe
 * and load a new problem.
 *
 * @author Michel Galley
 * @see Agent
 * @see BrokerGUI
 * @version 1.0 00/02/04
 */

class AgentGUI extends JFrame 
{    
    Agent agent;
    Platform platform;

    JButton bExit = new JButton("Exit");
    JButton bSubscribe = new JButton("Subscribe");
    JButton bUnsubscribe = new JButton("Unsubscribe");
    JButton bReset = new JButton("Problem Reset");
    JButton bLoad;
    JFileChooser fileLoader;
    JLabel label;
    JLabel fileLabel;
    String sLabel="Last loaded file : ";
    String fixFileLab="   ";
    String noneLabel="none                                    ";
    String fLabel;

    /**
     * Initializes a newly created AgentGUI object.
     *
     * @param  agent The agent owning this GUI.
     */
    AgentGUI(Agent agent)
    {
	super(agent.getID().getName());
	this.agent = agent;
	/**
	 * The use of the Platform object enables this object 
	 * not to have to do sJava synchronous calls on the agent.
	 */
	platform = new Platform(agent);
       
	try { UIManager.setLookAndFeel
		  (UIManager.getCrossPlatformLookAndFeelClassName());
	} catch (Exception e) {}

	bExit.setMnemonic(KeyEvent.VK_X);
	bExit.addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent e) {
		// Quitting, so unsubscribes to the broker
		platform.unsubscribe();
	    }
	});

	bSubscribe.setMnemonic(KeyEvent.VK_S);
	bSubscribe.addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent e) {
		// Subscribes to the broker
		platform.subscribe();
	    }
	});

	bUnsubscribe.setMnemonic(KeyEvent.VK_U);
	bUnsubscribe.addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent e) {
		// Unsubscribes to the broker
		platform.unsubscribe();
	    }
	});

	bReset.setMnemonic(KeyEvent.VK_S);
	bReset.addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent e) {
		fileLabel.setText(fixFileLab+noneLabel);
	    }
	});
       
	fileLoader=new JFileChooser("./");
	fileLoader.setApproveButtonText("Load");
	fileLoader.setApproveButtonMnemonic(KeyEvent.VK_L);
	fileLoader.setApproveButtonToolTipText("Load the selected problem file");
	javax.swing.filechooser.FileFilter problemFilter=new ProblemFilter();
	fileLoader.addChoosableFileFilter(problemFilter);
	fileLoader.setFileFilter(problemFilter);

        bLoad = new JButton("Load");
       	bLoad.setMnemonic(KeyEvent.VK_L);
	bLoad.addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent e) {
		System.out.println("Loading...");
		fileLoader.setApproveButtonText("Load");
		fileLoader.setApproveButtonMnemonic(KeyEvent.VK_L);
		int returnVal = fileLoader.showOpenDialog(AgentGUI.this);
		if (returnVal == JFileChooser.APPROVE_OPTION) {
		    File file = fileLoader.getSelectedFile();

		    if(platform.setProblem(file)){
			fLabel=file.getName();
			fileLabel.setText(fixFileLab+fLabel);
		    }
		    System.out.println("Opening: " + file.getName() + ".");
		} else {
		    System.err.println("Open command cancelled by user.");
		}
	    }
	});

	JPanel pane = new JPanel();
	pane.setBorder(BorderFactory.createEmptyBorder(10, 30, 10, 30));
	pane.setLayout(new GridLayout(4, 1));
	pane.add(bSubscribe);
	pane.add(bUnsubscribe);
	pane.add(bReset);
	pane.add(bExit);
	
	JPanel fpane = new JPanel();
	fpane.setBorder(BorderFactory.createEmptyBorder(30, 30, 0, 30));
	fpane.setLayout(new BorderLayout());
	fpane.add(bLoad,BorderLayout.SOUTH);
	
	label=new JLabel(sLabel);
	fileLabel = new JLabel(fixFileLab+noneLabel);
	fpane.add(label, BorderLayout.NORTH);
	fpane.add(fileLabel, BorderLayout.CENTER);

	getContentPane().add(fpane,BorderLayout.NORTH);
	getContentPane().add(pane,BorderLayout.CENTER);
	addWindowListener(new WindowAdapter() {
	    public void windowClosing(WindowEvent e) {
		platform.unsubscribe();
	    }
	});

	pack();
	setVisible(true);
    }
} 
