/*
The package platform is a semester project by Galley, Michel,
that Silaghi, Marius-Calin adviced in Spring 2000.
Several modifications were done subsequently by Silaghi, Marius-Calin.
Copyright (c) 2000, 2001, 2002, 2004
*/
/*
 * @(#)AgentID.java  1.0 00/02/04
 *
 * Intelligent Agents Collaborating in Distributed Problem Solving
 *
 * Supervisors:   Marius-Calin Silaghi, Djamila Sam-Haroud
 * Author:        Michel Galley
 */

package platform;

import java.lang.*;
import java.net.*;
import java.io.*;

/**
 * This class is used to identify Broker and Agent objects. It holds
 * the following information on it's holder :
 * <ul>
 * <li>IP address
 * <li>port number
 * <li>owner's name
 * </ul>
 *
 * @author Michel Galley
 * @see GenericAgent
 * @see Broker
 * @see Agent
 * @version 1.0 00/02/04
 */

public class AgentID implements Serializable{
    /** @serial Agent's name. */
    private String name;
    /** @serial Identifies whether or not the genericAgent is a broker. */
    private boolean isBroker;
    /** @serial Agent's host name. */
    private InetAddress host;
    /** @serial Agent's port number. */
    private int port;

    /**
     * Initializes a newly created AgentID object.
     *
     * @param  name Owner's name
     * @param  isBroker Indicates whether the owner is a broker or not
     * @param  host IP address
     * @param  port Port number
     */
    public AgentID(String name, boolean isBroker, InetAddress host, int port){
	this.name = name;
	this.isBroker = isBroker;
	this.host = host;
	this.port = port;
    }

    /****************
     * Public methods
     ****************/

    /** Return agent's name */
    public String getName() { return name; }
    /** Indicates whether the owner is a broker or not. */
    public boolean isBroker() { return isBroker; }
    /** Return the IP address of the agent. */
    public InetAddress getHost() { return host; }
    /** Return the port number of the agent. */
    public int getPort() { return port; }
    /** Return a String representation of the object. */
    public String toString() { return name; }

    /**
     * Compares the IP addresses and the port numbers of the two objects.
     *
     * @return Result of the comarison
     */
    public boolean equals(Object object){
	if(object instanceof AgentID) {
	    return ((host.toString().equals
		     (((AgentID)object).host.toString()))
		    && (port == ((AgentID)object).port));
	}
	return super.equals(object);
    }
    public int hashCode(){
      return host.hashCode()+port;
    }
}
