/*
The package platform is a semester project by Galley, Michel,
that Silaghi, Marius-Calin adviced in Spring 2000.
Several modifications were done subsequently by Silaghi, Marius-Calin.
Copyright (c) 2000, 2001, 2002, 2004
*/
/*
 * @(#)BrokerChildGUI.java  1.0 00/02/04
 *
 * Intelligent Agents Collaborating in Distributed Problem Solving
 *
 * Supervisors:   Marius-Calin Silaghi, Djamila Sam-Haroud
 * Author:        Michel Galley
 */

package platform;

import javax.swing.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;

/**
 * This class defines a new swing component extending JFrame. It is a child
 * window of the main broker window. It contains information specific to an
 * agent:
 * <ul>
 * <li>Matrix objects representing constraints defined in the agent's problem :
 * <p>The user can select variables in GUI and, if there is a constraint shared
 * by these two variables, the matrix representing this constraint is shown on
 * the GUI. Please refer to Matrix and MatrixModel classes for further
 * explainations.<p>
 * <li>Messages send by this agent or to this agent (shown in a JList
 * component) :
 * <p>The message area uses an ChildMessageList object as a model. Please
 * refer to the ChildMessageList class for further explainations.<p>
 * </ul>
 *
 * @author Michel Galley
 * @see BrokerGUI
 * @see ChildMessageList
 * @see Matrix
 * @see MatrixModel
 * @version 1.0 00/02/04
 */

class BrokerChildGUI extends JFrame{
    AgentID agentID;
    ChildMessageList model;

    /**
     * Swing components
     */

    JPanel pane = new JPanel();
    JPanel upPane = new JPanel();
    JPanel downPane = new JPanel();
    JPanel tablePane = new JPanel();
    JPanel combosPane = new JPanel();
    JPanel messagesPane = new JPanel();

    Matrix table;
    Hashtable tables = new Hashtable();
    Container cl;

    JLabel lAgentName = new JLabel();
    JLabel lNull = new JLabel("");
    //JLabel lVar1 = new JLabel("Variable1:");
    //JComboBox cVar1;
    //JLabel lVar2 = new JLabel("Variable2:");
    //JComboBox cVar2;
    JLabel lConstr = new JLabel("Constraints:");
    JComboBox cConstr;
    JButton bShow = new JButton("Show");
    JButton bShowAll = new JButton("ShowAll");
    JButton bRemove = new JButton("Remove");
    String[] newVars;

    JLabel lTitle = new JLabel("From     To       CID  MID  Message");
    JList messagesArea;
    JToolBar toolBar = new JToolBar(SwingConstants.VERTICAL);
    JButton button1 = new JButton(new ImageIcon("images/up.gif"));
    JButton button2 = new JButton(new ImageIcon("images/down.gif"));

    /**
     * Initializes a newly created BrokerChildGUI object.
     *
     * @param  _agentID Agent's ID
     * @param  _model ListModel object used as a model by the JList component
     */
    BrokerChildGUI(AgentID _agentID,
		   ChildMessageList _model){
        super(_agentID.getName());
	agentID = _agentID;
	model = _model;

	try { UIManager.setLookAndFeel
		  (UIManager.getCrossPlatformLookAndFeelClassName());
	} catch (Exception e) {}

	pane.setBorder(BorderFactory.createEmptyBorder(5, 10, 5, 10));
	pane.setLayout(new BoxLayout(pane, BoxLayout.Y_AXIS));

	upPane.setBorder(BorderFactory.createEmptyBorder(0, 0, 5, 0));
	//upPane.setLayout(new BoxLayout(upPane, BoxLayout.X_AXIS));
	upPane.setLayout(new BorderLayout());

	downPane.setLayout(new BoxLayout(downPane, BoxLayout.X_AXIS));

	// ComboBoxes
	combosPane.setBorder(BorderFactory.createEmptyBorder(0, 0, 0, 10));
	combosPane.setLayout(new GridLayout(8,1));
	combosPane.setMinimumSize(new Dimension(140, 140));
	combosPane.setPreferredSize(new Dimension(140, 140));
	combosPane.setMaximumSize(new Dimension(140, 140));

	cConstr = new JComboBox((MutableComboBoxModel)model
			      .getConstraintListModel().clone());
	//combosPane.setLayout(new GridLayout(7,1));
	//combosPane.setMinimumSize(new Dimension(100, 140));
	//combosPane.setPreferredSize(new Dimension(100, 140));
	//combosPane.setMaximumSize(new Dimension(100, 140));

	//cVar1 = new JComboBox((MutableComboBoxModel)model.getVarListModel(0));
	//cVar2 = new JComboBox((MutableComboBoxModel)model.getVarListModel(1));

	bShow.addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent e) {
                String sel = (String)cConstr.getSelectedItem();
                if(sel==null) return;
		MatrixModel matrixModel
		    = model.getMatrixModel(sel);
		//= model.getMatrixModel((String)cVar1.getSelectedItem(),
		//(String)cVar2.getSelectedItem());
                //setTitle(agentID.getName()+" "+(String)cConstr.getSelectedItem());
                if(tables.get(matrixModel)!=null) return;
         	table = new Matrix(null);
	        table.setBorder(BorderFactory.createLineBorder(Color.black));
		table.setModel(matrixModel);
                cl.add((JPanel)table);
                tables.put(matrixModel,table);
                pack();
	    }
	});

	bShowAll.addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent e) {
              for(int i=0;i<cConstr.getItemCount();i++){
		MatrixModel matrixModel
		    = model.getMatrixModel((String)cConstr.getItemAt(i));
                if(matrixModel==null) continue;
                if(tables.get(matrixModel)!=null) continue;
         	table = new Matrix(null);
	        table.setBorder(BorderFactory.createLineBorder(Color.black));
		table.setModel(matrixModel);
                tables.put(matrixModel,table);
                cl.add((JPanel)table);
              }
              pack();
	    }
	});

	bRemove.addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent e) {
                String sel=(String)cConstr.getSelectedItem();
                if(sel==null) return;
		MatrixModel matrixModel
		    = model.getMatrixModel(sel);
                if(matrixModel==null) return;
                table = (Matrix)tables.get(matrixModel);
                if(table==null) return;
                cl.remove((JPanel)table);
                tables.remove(matrixModel);
                pack();
	    }
	});

	lAgentName.setText(agentID.toString());
	combosPane.add(lAgentName);
	combosPane.add(lNull);
	//combosPane.add(lVar1);
 	//combosPane.add(cVar1);
 	//combosPane.add(lVar2);
 	//combosPane.add(cVar2);
	combosPane.add(lConstr);
 	combosPane.add(cConstr);
	combosPane.add(lNull);
	combosPane.add(bShowAll);
	combosPane.add(bShow);
	combosPane.add(bRemove);

	// table
        FlowLayout flowLayout = new FlowLayout();
        cl = new Container();
        cl.setLayout(flowLayout);

        /*
	table = new Matrix(null);
	table.setBorder(BorderFactory.createLineBorder(Color.black));
        */
	upPane.add(combosPane, BorderLayout.WEST);
        //cl.add((JPanel)table);
	upPane.add(cl, BorderLayout.CENTER);

	// Messages
	messagesPane.setLayout(new BorderLayout());
	messagesPane.setMinimumSize(new Dimension(0, 114));
	messagesPane.setPreferredSize(new Dimension(220, 114));
	messagesPane.setMaximumSize(new Dimension(1000, 114));
	messagesPane.setBorder
	    (BorderFactory.createLineBorder(Color.black));
	messagesPane.setLayout(new BorderLayout());
	toolBar.setFloatable(false);

	button1.addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent e) {
		((ChildMessageList)messagesArea.getModel())
		    .moveSelection(-1);
		messagesArea.setSelectedIndex(2);
	    }
	});
	button2.addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent e) {
		((ChildMessageList)messagesArea.getModel())
		    .moveSelection(1);
		messagesArea.setSelectedIndex(2);
	    }
	});

	toolBar.add(button1);
	toolBar.add(button2);

	messagesArea = new JList((ListModel)model);

	lTitle.setFont(new Font("Monospaced", Font.PLAIN, 12));
	messagesArea.setFont(new Font("Monospaced", Font.PLAIN, 12));
	messagesArea.setSelectedIndex(2);

	messagesPane.add(lTitle, BorderLayout.NORTH);
	messagesPane.add(messagesArea, BorderLayout.CENTER);

	downPane.add(messagesPane);
	downPane.add(toolBar);

	pane.add(upPane);
	pane.add(downPane);
	getContentPane().add(pane);

	pack();
	setVisible(true);
    }
}
