/*
The package platform is a semester project by Galley, Michel,
that Silaghi, Marius-Calin adviced in Spring 2000.
Several modifications were done subsequently by Silaghi, Marius-Calin.
Copyright (c) 2000, 2001, 2002, 2004
*/
/*
 * @(#)BrokerGUI.java  1.0 00/02/04
 *
 * Intelligent Agents Collaborating in Distributed Problem Solving
 *
 * Supervisors:   Marius-Calin Silaghi, Djamila Sam-Haroud
 * Author:        Michel Galley
 */

package platform;

import java.net.*;
import java.util.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import utils.*;


/**
 * This class defines a new swing component extending JFrame. It is the
 * broker's main window. It contains a JList component (the message area)
 * showing all the messages send by or sent to the broker. This component
 * uses a MessageList object as a model (ListModel).
 * <p>The GUI also enables the user to select a specific agent that will
 * lead a new window to open and show specific information about the this
 * agent. This new window is a BrokerChildGUI object.
 *
 * @author Michel Galley
 * @see MessageList
 * @see BrokerChildGUI
 * @version 1.0 00/02/04
 */


class BrokerGUI extends JFrame implements KeyListener{
    /* Variables: */

    Broker broker;
    Platform platform;
    MessageList model;
    Vector childWindows;
    int SELECTED_ITEM;

    JPanel leftPane = new JPanel();
    JPanel rightPane = new JPanel();
    JPanel buttonsPane = new JPanel();
    JPanel combosPane = new JPanel();
    JPanel messagesPane = new JPanel();
    JPanel messageBoxPane = new JPanel();
    JPanel netinfoPane = new JPanel();
    JPanel hostPane = new JPanel();
    JPanel portPane = new JPanel();

    JComboBox cProtocol = new JComboBox();
    JLabel lProtocol = new JLabel();
    //static final String nDefaultp = "Default";

    JComboBox cAgent = new JComboBox();
    JLabel lAgent = new JLabel();
    static final String noAgent = "choose...";

    JButton bStart = new JButton("Start Search");
    JButton bStop = new JButton("Stop Search");
    JButton bClear = new JButton("Clear");
    JButton bExit = new JButton("Exit");

    JLabel hostArea = new JLabel();
    JLabel portArea = new JLabel();

    JLabel lHost = new JLabel();
    JLabel lPort = new JLabel();

    JLabel lTitle = new JLabel("From     To       CID  MID  Message");
    JList messagesArea;
    JLabel lStatus = new JLabel("No active protocol                 ");
    JButton button1 = new JButton("up");//new ImageIcon("images/up.gif"));
    JButton button2 = new JButton("lo");//new ImageIcon("images/down.gif"));
    JToolBar toolBar = new JToolBar(SwingConstants.VERTICAL);
    String status;

    AgentID[] agentIDs;
    Vector protocols;

    /**
     * Initializes a newly created BrokerGUI object.
     *
     * @param  _broker Broker's ID
     * @param  host Broker's IP address
     * @param  port Broker's port number
     * @param  _model ListModel object used as a model by the JList component
     */
    BrokerGUI(Broker _broker, InetAddress host, int port,
	      MessageList _model){
	super("MELY");
	SELECTED_ITEM=MessageList.VISIBLE_SIZE/2;
	broker = _broker;
	model = _model;
	platform = new Platform(broker);
	childWindows = new Vector(0);

	try { UIManager.setLookAndFeel
		  (UIManager.getCrossPlatformLookAndFeelClassName());
	} catch (Exception e) {}

	leftPane.setBorder(BorderFactory.createEmptyBorder(10, 20, 10, 10));
	leftPane.setLayout(new BorderLayout());

	rightPane.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 20));
	rightPane.setLayout(new BoxLayout(rightPane, BoxLayout.Y_AXIS));

	// ComboBoxes
	// Protocols
	combosPane.setBorder(BorderFactory.createEmptyBorder(10, 10, 0, 10));
	combosPane.setLayout(new GridLayout(4, 1));

	cProtocol.setEditable(false);
	//cProtocol.addItem(nDefaultp);
	cProtocol.addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent e) {
		if(cProtocol.getSelectedItem() != null)
		    if(!platform.chooseProtocol
		       (cProtocol.getSelectedItem().getClass()))
			cProtocol.setSelectedIndex(0);
	    }
	});
	lProtocol.setText("Protocol :");
	combosPane.add(lProtocol);
	combosPane.add(cProtocol);

	// Agents
	cAgent.addItem(noAgent);
	cAgent.addItemListener(new ItemListener() {
	    public void itemStateChanged(ItemEvent e) {
		if(e.getStateChange() == ItemEvent.SELECTED)
		    if(e.getItem() instanceof AgentID) {
			ChildMessageList childModel =
			    model.getChildMessageList((AgentID)e.getItem());
			if(childModel != null)
			    new BrokerChildGUI((AgentID)e.getItem(),
					       childModel);
			cAgent.setSelectedIndex(0);
		    }
	    }
	});
	lAgent.setText("Agent :");
	combosPane.add(lAgent);
	combosPane.add(cAgent);
	leftPane.add(combosPane, BorderLayout.NORTH);


	// Buttons
	buttonsPane.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));
	buttonsPane.setLayout(new GridLayout(4, 1));

	bStart.setMaximumSize(new Dimension(60, 20));
	bStart.setMnemonic(KeyEvent.VK_S);
	bStart.addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent e) {
		platform.startProtocol(null); }
	});

	bStop.setMaximumSize(new Dimension(60, 20));
	bStop.setMnemonic(KeyEvent.VK_P);
	bStop.addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent e) {
		platform.endProtocol(); }
	});

	bClear.setMaximumSize(new Dimension(60, 20));
	bClear.setMnemonic(KeyEvent.VK_C);
	bClear.addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent e) {
		model.clear(); }
	});

	bExit.setMaximumSize(new Dimension(60, 20));
	bExit.setMnemonic(KeyEvent.VK_X);
	bExit.addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent e) {
		platform.brokerShutsDown(); }
	});

	buttonsPane.add(bStart);
	buttonsPane.add(bStop);
	buttonsPane.add(bClear);
	buttonsPane.add(bExit);
	leftPane.add(buttonsPane, BorderLayout.SOUTH);

	// Host and port
	netinfoPane.setLayout(new GridLayout(1,2));
	netinfoPane.setBorder(BorderFactory.createEmptyBorder(10, 0, 10, 0));
	lHost.setText("Host: "+host.getHostName());
	lPort.setText("Port: "+Integer.toString(port));
	netinfoPane.add(lHost);
	netinfoPane.add(lPort);

	// Text area
 	messagesArea = new JList(model);

	messagesPane.setLayout(new BoxLayout(messagesPane,
					     BoxLayout.X_AXIS));
	messageBoxPane.setBorder
	    (BorderFactory.createLineBorder(Color.black));
	messageBoxPane.setMinimumSize(new Dimension(0, 226));
	messageBoxPane.setPreferredSize(new Dimension(500, 326));
	messageBoxPane.setMaximumSize(new Dimension(2000, 426));
	messageBoxPane.setLayout(new BorderLayout());

	lTitle.setFont(new Font("Monospaced", Font.PLAIN, 12));
	messagesArea.setFont(new Font("Monospaced", Font.PLAIN, 12));
	lStatus.setFont(new Font("Monospaced", Font.PLAIN, 12));
	messagesArea.setPrototypeCellValue(new String(new byte[80]));
	messagesArea.setSelectedIndex(SELECTED_ITEM);

	button1.addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent e) {
		((MessageList)messagesArea.getModel()).moveSelection(-1);
		messagesArea.setSelectedIndex(SELECTED_ITEM);
	    }
	});
	button2.addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent e) {
		((MessageList)messagesArea.getModel()).moveSelection(1);
		messagesArea.setSelectedIndex(SELECTED_ITEM);
	    }
	});

	addKeyListener(this);
	button1.addKeyListener(this);
	button2.addKeyListener(this);
	messagesArea.addKeyListener(this);
	messageBoxPane.addKeyListener(this);
	toolBar.addKeyListener(this);
	leftPane.addKeyListener(this);
	rightPane.addKeyListener(this);
	getContentPane().addKeyListener(this);
	messagesPane.addKeyListener(this);

	toolBar.setFloatable(false);
	toolBar.add(button1);
	toolBar.add(button2);

	messageBoxPane.add(lTitle, BorderLayout.NORTH);
	messageBoxPane.add(messagesArea, BorderLayout.CENTER);
	messageBoxPane.add(lStatus, BorderLayout.SOUTH);

	messagesPane.add(messageBoxPane);
	messagesPane.add(toolBar);

	rightPane.add(netinfoPane);
	rightPane.add(messagesPane);

	getContentPane().add(leftPane, BorderLayout.WEST);
	getContentPane().add(rightPane, BorderLayout.CENTER);

	addWindowListener(new WindowAdapter() {
	    public void windowClosing(WindowEvent e) {
		platform.brokerShutsDown(); }
	});
	pack();
	setVisible(true);
    }

    /** Update the agent list contained in the agent ComboBox */
    synchronized void updateAgents(AgentID[] _agentIDs)
    {
	agentIDs = _agentIDs;
	SwingUtilities.invokeLater(new Runnable() {
	    public void run() {
		cAgent.removeAllItems();
		cAgent.addItem(noAgent);
		for(int index=0; index<agentIDs.length; index++){
		    cAgent.addItem(agentIDs[index]);
		    MySystem.err.println("added agent "+agentIDs[index]);
		}
	    }
	});
    }

    /** Update the protocol list contained in the protocol ComboBox */
    synchronized void updateProtocols(Vector _protocols)
    {
	protocols = _protocols;
	SwingUtilities.invokeLater(new Runnable() {
	    public void run() {
		cProtocol.removeAllItems();
		//cProtocol.addItem(nDefaultp);

		for(int index=0; index<protocols.size(); index++) {
		    try { cProtocol.addItem(((Class)protocols.elementAt(index))
					    .newInstance()); }
		    catch(Exception e) { e.printStackTrace();
		    System.exit(-1); }
		}
	    }
	});
    }

    /** Update the message in the status bar */
    void updateProtocolStatus(String newStatus)
    {
	status = newStatus;
	SwingUtilities.invokeLater(new Runnable() {
	    public void run() {
		lStatus.setText(status); }
	});
    }

    public void keyTyped(KeyEvent e){
    }
    public void keyPressed(KeyEvent e){
	    if(e.getKeyCode()==KeyEvent.VK_UP){
		((MessageList)messagesArea.getModel()).moveSelection(-1);
		messagesArea.setSelectedIndex(SELECTED_ITEM);
	    }else if(e.getKeyCode()==KeyEvent.VK_DOWN){
		((MessageList)messagesArea.getModel()).moveSelection(1);
		messagesArea.setSelectedIndex(SELECTED_ITEM);
	    }else if(e.getKeyCode()==KeyEvent.VK_PAGE_UP){
		((MessageList)messagesArea.getModel()).moveSelection(-MessageList.VISIBLE_SIZE);
		messagesArea.setSelectedIndex(SELECTED_ITEM);
	    }else if(e.getKeyCode()==KeyEvent.VK_PAGE_DOWN){
		((MessageList)messagesArea.getModel()).moveSelection(MessageList.VISIBLE_SIZE);
		messagesArea.setSelectedIndex(SELECTED_ITEM);
	    }
    }
    public void keyReleased(KeyEvent e){
    }
}

