/*
The package platform is a semester project by Galley, Michel,
that Silaghi, Marius-Calin adviced in Spring 2000.
Several modifications were done subsequently by Silaghi, Marius-Calin.
Copyright (c) 2000, 2001, 2002, 2004
*/
/*
 * @(#)LogFile.java  1.0 00/02/05
 *
 * Intelligent Agents Collaborating in Distributed Problem Solving
 *
 * Supervisors:   Marius-Calin Silaghi, Djamila Sam-Haroud
 * Author:        Michel Galley
 */

package platform;

import java.io.*;
import java.util.*;
import utils.*;

/**
 * Class defining primities enabling the MessageList object to read and
 * write messages on a file, without having to worry about file access and
 * file streams.
 *
 * @author Michel Galley
 * @see Message
 * @see MessageList
 * @version 1.0 00/02/05
 */

class LogFile
{
    private static final String FILE_NAME = ".history.mhc";
    private static final int AVG_SIZE = 30;

    FileInputStream fileIn = null;
    FileOutputStream fileOut = null;
    DataInputStream in = null;
    DataOutputStream out = null;

    int currentMessage = 0;
    long currentPosition = 0;

    /**
     * Initializes a newly created LogFile object. The file is cleared.
     */
    LogFile() { clear(); }

    private void read_mode()
    {
	try {
	    close_file();
	    fileIn = new FileInputStream(FILE_NAME);
	    in = new DataInputStream(fileIn);
	}
	catch(IOException e) { e.printStackTrace(); System.exit(-1); }
     }

    private void write_mode()
    {
	try {
	    if(fileOut == null) {
		close_file();
		fileOut = new FileOutputStream(FILE_NAME, true);
		out = new DataOutputStream(fileOut);
	    }
	}
	catch(IOException e) { e.printStackTrace(); System.exit(-1); }
    }

    private void close_file()
    {
	try {
	    if(fileOut != null) {
		out.close();
		fileOut.close();
	    }
	    fileOut = null;
	    if(fileIn != null) {
		in.close();
		fileIn.close();
	    }
	    fileIn = null;
	}
	catch(IOException e) { e.printStackTrace(); System.exit(-1); }
    }

    /** Delete the file if it exists. */
    synchronized void clear()
    {
	close_file();
	File file = new File(FILE_NAME);
	if(file.exists()) file.delete();
    }

    /**
     * Fill the buffer as much as it can.
     *
     * @param  buffer The buffer to fill
     * @param  bufferStart The position where to start to fill
     * @return The number of messages writted in the buffer
     */
    synchronized int fillBuffer(Message[] buffer, int bufferStart){
	int i=0;

	try {
	    read_mode();
	    Message.skipMessages(in, bufferStart);
	    // Fill as much as it can
	    while(i<buffer.length) {
		buffer[i] = Message.readMessage(in);
		if(buffer[i]==null){
		    if(i>0){
			MySystem.err.println("Error reading:"+
					     buffer[i-1]);
			throw new Exception("error reading message");
		    }else{
			MySystem.err.println("Error reading at:"+
					     bufferStart);
			throw new Exception("error reading message");
		    }
		}
		i++;
		//System.out.println(i-1+": "+buffer[i-1]);
	    }
	} catch(Exception e){
	    if(!(e instanceof EOFException)) {
		e.printStackTrace();
		System.exit(-1);
	    }
	}
	//System.out.println("Read "+i+" messages.");
	return i-1;
    }

    /** Append a new message at the end of the file. */
    synchronized void putMessage(Message message)
    {
	write_mode();
	try {
	    //out.writeObject(message);
	    message.writeMessage(out);
	}
	catch(IOException e)
	{
	    if(e.getMessage().equals("Bad file number"))
		e.printStackTrace();
	}
    }
}

