/*
The package platform is a semester project by Galley, Michel,
that Silaghi, Marius-Calin adviced in Spring 2000.
Several modifications were done subsequently by Silaghi, Marius-Calin.
Copyright (c) 2000, 2001, 2002, 2004
*/
/*
 * @(#)Matrix.java  1.0 00/02/04
 *
 * Intelligent Agents Collaborating in Distributed Problem Solving
 *
 * Supervisors:   Marius-Calin Silaghi, Djamila Sam-Haroud
 * Author:        Michel Galley
 */

package platform;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

/**
 * This class is a swing component extending JPanel, giving a graphical
 * representation of a matrix. It is filled with colored cells.
 *
 * @author Michel Galley
 * @see MatrixModel
 * @see BrokerChildGUI
 * @version 1.0 00/02/04
 */

class Matrix extends JPanel
{

    private final Color bg = Color.white; //Color.lightGray;
    private final Color fg = Color.black;

    private MatrixModel model;

    /**
     * Initializes a newly created Matrix object. It uses a MatrixModel
     * object as a model.
     *
     * @param  model The MatrixModel object.
     */
    Matrix(MatrixModel model)
    {
	setBackground(bg);
	setForeground(fg);
 	setPreferredSize(new Dimension(110, 110));
	setBorder(BorderFactory.createLineBorder(Color.black, 1));
    }

    /**
     * Give a new model to the matrix.
     *
     * @param  model The new model.
     */
    void setModel(MatrixModel model)
    {
	if(this.model != null)
	    this.model.removeMatrixListener(this);
	this.model = model;
	if(model != null)
	    model.addMatrixListener(this);

	repaint();
    }

    /** Return the model the matrix uses. */
    MatrixModel MatrixModel() { return model; }

    /**
     * If the UI delegate is non-null, call its paint method. We pass the
     * delegate a copy of the Graphics object to protect the rest of the paint
     * code from irrevocable changes (e.g. Graphics.translate()).
     */
    public void paintComponent(Graphics g)
    {
	super.paintComponent(g);
	if(model == null) return;

	int sizeX = model.getSizeX(), sizeY = model.getSizeY();

	if(sizeX == 0 || sizeY == 0)
	    return;

        Insets insets = getInsets();
        int currentWidth = getWidth() - insets.left - insets.right;
        int currentHeight = getHeight() - insets.top - insets.bottom;
        float gridWidth = (float)currentWidth / (float)(sizeX+3);
        float gridHeight = (float)currentHeight / (float)(sizeY+3);

	// Draw matrix cells
	for(int x=0; x<sizeX; x++)
	    for(int y=0;y<sizeY; y++) {
		g.setColor(model.getColor(x,y));
		g.fillRect((int)((x+1)*gridWidth+1), (int)((y+1)*gridHeight+1),
			   (int)gridWidth,(int) gridHeight);
	    }

	// Draw column and line cells
	for(int x=0; x<sizeX; x++) {
	    g.setColor(model.getColor(x, sizeY));
	    g.fillRect((int)((x+1)*gridWidth+1), (int)((sizeY+2)*gridHeight+1),
		       (int)gridWidth,(int) gridHeight);
	}

	for(int y=0; y<sizeY; y++) {
	    g.setColor(model.getColor(sizeX, y));
	    g.fillRect((int)((sizeX+2)*gridWidth+1), (int)((y+1)*gridHeight+1),
		       (int)gridWidth,(int) gridHeight);
	}

	// Draw lines
	g.setColor(Color.black);
	for(int x=0; x<=sizeX+2; x++)
	    g.drawLine((int)((x+1)*gridWidth), (int)gridHeight,
		       (int)((x+1)*gridWidth), (int)((sizeY+1)*gridHeight));
	for(int y=0; y<=sizeY+2; y++)
	    g.drawLine((int)gridWidth, (int)((y+1)*gridHeight),
		       (int)((sizeX+1)*gridWidth), (int)((y+1)*gridHeight));

	for(int x=0; x<=sizeX; x++)
	    g.drawLine((int)((x+1)*gridWidth), (int)((sizeY+2)*gridHeight),
		       (int)((x+1)*gridWidth), (int)((sizeY+3)*gridHeight));

	for(int y=0; y<=sizeY; y++)
	    g.drawLine((int)((sizeX+2)*gridWidth), (int)((y+1)*gridHeight),
		       (int)((sizeX+3)*gridWidth), (int)((y+1)*gridHeight));

	// Draw text
	g.setFont(new Font("Monospaced", Font.PLAIN, 8));
	for(int x=0; x<sizeX; x++)
	    g.drawString(""+(x), (int)((x+1.2)*gridWidth),
			 (int)(0.8*gridHeight));
	for(int y=0; y<sizeY; y++)
	    g.drawString(""+(y), (int)(0.2*gridWidth),
			 (int)((y+1.8)*gridHeight));
	g.setFont(new Font("Monospaced", Font.PLAIN, 8));
	g.drawString("("+model.id+") "+model.getVariable1()+" x "+model.getVariable2(),
		     (int)((sizeX/2.-1.5)*gridWidth),
		     (int)((sizeY+1.8)*gridHeight));
	g.drawString(model.getVariable2(),
		     (int)((sizeX+1.2)*gridWidth),
		     (int)((sizeY+2.8)*gridHeight));
	g.drawString(model.getVariable1(),
		     (int)((sizeX+2.0)*gridWidth), //2.2
		     (int)((sizeY+1.75)*gridHeight));
    }
}
