/*
The package platform is a semester project by Galley, Michel,
that Silaghi, Marius-Calin adviced in Spring 2000.
Several modifications were done subsequently by Silaghi, Marius-Calin.
Copyright (c) 2000, 2001, 2002, 2004
*/
/*
 * @(#)MatrixModel.java  1.0 00/02/05
 *
 * Intelligent Agents Collaborating in Distributed Problem Solving
 *
 * Supervisors:   Marius-Calin Silaghi, Djamila Sam-Haroud
 * Author:        Michel Galley
 */

package platform;

import java.awt.*;
import utils.*;

/**
 * The <code>MatrixModel</code> class represents a matrix whose elements
 * are colors.
 * <p>Matrix objects can register as a listener to MatrixModel objects, and
 * be notified as their models change.
 *
 * @author Michel Galley
 * @see Matrix
 * @see ChildMessageList
 * @version 1.0 00/02/05
 */

class MatrixModel{
    String var1, var2;
    Color[][] matrix;
    /** No more than one listener. */
    Matrix listener;
    int id;

    /**
     * Initialized a newly created MatrixModel object. A model is created
     * each time a createColorMatrix message is received by the broker
     * (it is passed via MessageList and ChildMessageList objects).
     *
     * @param  message The message causing the creation of the matrix
     */
    MatrixModel(Message_CreateColorMatrix message){
	var1 = message.getVariable1();
	var2 = message.getVariable2();
	matrix = new Color[message.getSizeY()+1][message.getSizeX()+1];
        id=message.id;
	clear();
    }

    /** Add a new Matrix listener to this model. */
    public void addMatrixListener(Matrix listener){
	this.listener = listener;
	listener.repaint();
    }

    /** Remove th Matrix listener from this model. */
    public void removeMatrixListener(Matrix listener){
	this.listener = null;
    }

    /**************************************
     * Functions to access member variables
     **************************************/

    String getVariable1() { return var1; }
    String getVariable2() { return var2; }
    int getSizeX() { return matrix.length-1; }
    int getSizeY() { return matrix[0].length-1; }
    Color getColor(int x, int y) { return matrix[x][y]; }

    /** Modify the matrix according to a setColor message. */
    void setColor(Message_SetColor setColor){
	if(setColor.domain!=null){
	    int x,y,xsize,ysize;
	    xsize=getSizeX();
	    //System.err.println("XSize="+xsize);
	    ysize = getSizeY();
	    //System.err.println("YSize="+ysize);
	    if(!setColor.getOldColor().equals(Utils.transparent))
		for(x=0;x<ysize;x++)
		    matrix[xsize][x] = setColor.getOldColor();
	    if(!setColor.getColor().equals(Utils.transparent))
		for(x=setColor.domain[0].getL();x<=setColor.domain[0].getU();x++)
		    matrix[xsize][x] = setColor.getColor();

	    if(!setColor.getOldColor().equals(Utils.transparent))
		for(x=0;x<xsize;x++)
		    matrix[x][ysize] = setColor.getOldColor();

	    if(!setColor.getColor().equals(Utils.transparent))
		for(x=setColor.domain[1].getL();x<=setColor.domain[1].getU();x++)
		    matrix[x][ysize] = setColor.getColor();

	    if(listener != null)
		listener.repaint();
	    return;
	}
	if(setColor.table==null){
	    for(int x=setColor.getPosX();x<=setColor.getPosX2();x++)
		for(int y=setColor.getPosY();y<=setColor.getPosY2();y++){
		    matrix[y][x] = setColor.getColor();
		    if(listener != null)
			listener.repaint();
		}
	    return;
	}
	for(int x=setColor.getPosX();x<=setColor.getPosX2();x++){
	    for(int y=setColor.getPosY();y<=setColor.getPosY2();y++){
		int x0=setColor.destX+x-setColor.getPosX(),y0=setColor.destY+y-setColor.getPosY();
		if(setColor.table[x][y]==1){
		    matrix[y0][x0] = setColor.getColor();
		    //MySystem.out.println("m["+x0+y0+"]=1<-"+setColor.getColor());
		}else{
		    matrix[y0][x0] = setColor.getColor0();
		    //MySystem.out.println("m["+x0+y0+"]=0<-"+setColor.getColor0());
		}
		if(listener != null)
		    listener.repaint();
	    }
	}
    }
    /** Modify the matrix according to a setColor message.*/
    void unsetColor(Message_SetColor setColor) {
	if(setColor.domain!=null){
	    int x,y,sizeX,sizeY;
	    Color color;

	    sizeX=getSizeX();
	    sizeY = getSizeY();
	    color=setColor.getOldColor();
	    if(!color.equals(Utils.transparent)){
		//if(setColor.oldDomain==null)
		//    color=Color.lightGray;

		for(x=0;x<sizeY;x++)
		    matrix[sizeX][x] = color;
		for(x=0;x<sizeX;x++)
		    matrix[x][sizeY] = color;
	    }
	    color=setColor.getColor();
	    if((setColor.oldDomain!=null)&&
	       (!setColor.getOldColor().equals(Utils.transparent))){
		for(x=setColor.oldDomain[0].getL();x<=setColor.domain[0].getU();x++)
		    matrix[sizeX][x] = color;
		for(x=setColor.oldDomain[1].getL();x<=setColor.domain[1].getU();x++)
		    matrix[x][sizeY] = color;
	    }
	    if((setColor.oldDomain!=null)&&
	       (setColor.getOldColor().equals(Utils.transparent))){
		color = Color.green;
		for(x=setColor.domain[0].getL();x<=setColor.domain[0].getU();x++)
		    matrix[sizeX][x] = color;
		for(x=setColor.domain[1].getL();x<=setColor.domain[1].getU();x++)
		    matrix[x][sizeY] = color;
	    }
	    if(listener != null)
		listener.repaint();
	    return;
	}

	if(setColor.table==null)
	    for(int x=setColor.getPosX();x<=setColor.getPosX2();x++)
		for(int y=setColor.getPosY();y<=setColor.getPosY2();y++){
		    matrix[y][x] = setColor.getOldColor();
		    if(listener != null)
			listener.repaint();
		}
	else
	    for(int x=setColor.getPosX();x<=setColor.getPosX2();x++)
		for(int y=setColor.getPosY();y<=setColor.getPosY2();y++){
		    int x0=setColor.destX+x-setColor.getPosX(),y0=setColor.destY+y-setColor.getPosY();
		    matrix[y0][x0] =
			(setColor.table[x][y]==1)?setColor.getOldColor():setColor.getOldColor0();
		    if(listener != null)
			listener.repaint();
		}
    }

    /** Clear the matrix. */
    void clear()
    {
	for(int x=0; x<matrix.length; x++)
	    for(int y=0; y<matrix[0].length; y++)
		matrix[x][y] = Color.lightGray;

	if(listener != null)
	    listener.repaint();
    }
}
