/*
The package platform is a semester project by Galley, Michel,
that Silaghi, Marius-Calin adviced in Spring 2000.
Several modifications were done subsequently by Silaghi, Marius-Calin.
Copyright (c) 2000, 2001, 2002, 2004
*/
/*
 * @(#)MessageQueue.java  1.0 00/02/04
 *
 * Intelligent Agents Collaborating in Distributed Problem Solving
 *
 * Supervisors:   Marius-Calin Silaghi, Djamila Sam-Haroud
 * Author:        Michel Galley,  Marius-Calin Silaghi
 */

package platform;

import sjavap.*;
import java.lang.*;
import java.util.*;
import utils.*;

class OneMessageQueue implements Runnable, Reply{
    public String name;
    private static final int SIZE_INC = 20;
    private static final int _STATIC_CALLS = 5;
    int _CALLS = 5;

    private ArrayList actionTriggers=new ArrayList();
    private MessageSelector[] types=new MessageSelector[0];
    private MessageReceiver[] receivers=new MessageReceiver[0];
    private int[] procedures=new int[0];

    private Vector messages;
    private int nbOfMessages;
    MessageReceiver receiver = null;
    boolean shuttingDown = false;

    public static final int _laddMessage = 1;
    public static final int _lsetReceiver = 2;
    public static final int _lclear = 3;
    public static final int _lshutDown = 4;
    public static final int _lsetActionTrigger = 5;

 /**     object support (needs implements Reply)               **/
 /*********************************************************************/
 /* Written by Claude Petitpierre,   LTI-DI-EPFL,           7 Apr 1999*/
 /*********************************************************************/
 /* Modif. by Marius-Calin Silaghi, LIA-DI-EPFL,silaghi@lia.di.epfl.ch*/
 /*  Bug that may lead to deadlock in Synch_Wait(),fixed,  30 Nov 1999*/
 /*********************************************************/
 /**/ Synch_List select = new Synch_List(this) ;          //
 /**/ synchronized public void Synch_Reply()              //
 /**/   { notify(); }                                     //
 /**/ int now() {return Kernel.kern.currentTime;}         //
 /**/ int Synch_Wait() {                                  //
 /**/   Kernel.kern.Synch_Request(select);                //
 /**/   do                                                //
 /**/     {try {wait();} catch(InterruptedException e){}} //
 /**/   while(select.nbEntries!=0);                       //
 /**/   return select.Value; }                            //
 /****************** Need not be modified *****************/


    public OneMessageQueue(MessageReceiver receiver, String name) {
	this.name=name;
	messages = new Vector(SIZE_INC);
	nbOfMessages = 0;
	this.receiver = receiver;
    }

    int getNbOfMessages(){
	return nbOfMessages;
    }

    void printMessages(){
	for(int i=0; i<nbOfMessages; i++){
	    MySystem.err.println("PMQ:"+i+":"+messages.get(i));
	}
	for(int a=0;a<actionTriggers.size();a++){
	    MySystem.err.println("PMQ:actionTriggers:"+a+":"+actionTriggers.get(a));
	}
    }

    synchronized public void laddMessage(Message message){
	messages.add(nbOfMessages, message);
	nbOfMessages++;
	if(nbOfMessages == messages.size())
	    messages.setSize(messages.size() + SIZE_INC);
	Synch_Reply();
    }

    synchronized public void lsetReceiver(MessageReceiver receiver){
	this.receiver = receiver;
	Synch_Reply();
    }

    synchronized public void lsetActionTrigger(ActionTrigger at){
	actionTriggers.add(at);
	Synch_Reply();
    }

    synchronized public void lclear(){
        types=new MessageSelector[0];
        receivers=new MessageReceiver[0];
        procedures=new int[0];

	messages.clear();
	nbOfMessages = 0;
	messages.setSize(SIZE_INC);
	actionTriggers=new ArrayList();
	Synch_Reply();
    }

    synchronized public void lshutDown(){
	shuttingDown = true;
	Synch_Reply();
   }

    synchronized public void lsetReceiver(MessageReceiver receiver, MessageSelector type){
	int size=receivers.length;
	MessageReceiver[] omr=receivers;
	MessageSelector[] oc=types;
	int[] op=procedures;
	receivers=new MessageReceiver[size+1];
	types=new MessageSelector[size+1];
	procedures=new int[size+1];
	for(int k=0;k<size;k++){
	    types[k]=oc[k];
	    receivers[k]=omr[k];
	    procedures[k]=op[k];
	}
	receivers[size] = receiver;
	types[size]=type;
	procedures[size]=-1;
	Synch_Reply();
    }

    synchronized public void lsetReceiver(MessageReceiver receiver, MessageSelector type, int procedureID){
	int size=receivers.length;
	MessageReceiver[] omr=receivers;
	MessageSelector[] oc=types;
	int[] op=procedures;
	receivers=new MessageReceiver[size+1];
	types=new MessageSelector[size+1];
	procedures=new int[size+1];
	for(int k=0;k<size;k++){
	    types[k]=oc[k];
	    receivers[k]=omr[k];
	    procedures[k]=op[k];
	}
	receivers[size] = receiver;
	types[size]=type;
	procedures[size]=procedureID;
	//should not be refused
	if(procedureID>=MessageQueue.REFUSE)
	    MySystem.exit(1);
	Synch_Reply();
    }

    private void choiceActionTrigger(int choice){
	ActionTrigger at=(ActionTrigger)actionTriggers.get(choice);
	at.actionAtChoice();
	if(at.removeAtChoice())
	    actionTriggers.remove(choice);
    }

    private int selectActionTriggers(){
	int selectID=0;
	Iterator it=actionTriggers.iterator();
	while(it.hasNext()){
	    ActionTrigger at=(ActionTrigger)it.next();
	    if(at.active())
		select.call(_STATIC_CALLS+selectID,at.getTarget(),at.getProcedureID(),at.getPriority());
	    selectID++;
	}
	return selectID;
    }

    private int getTypeIndex(Message message){
	for(int k=0; k<types.length; k++){
	    if (types[k].accepts((Message_Command)message))
		return k;
	}
	return -1;
    }
    boolean traced=false;
    synchronized public void run(){
	select.thread=Thread.currentThread();
	int reply, target;
	Message message;
	if(name.equals("protocolInQueue")){
	    traced=false;
	    //MySystem.out.println("protocolInQueue");
	}

	while(!shuttingDown || nbOfMessages != 0) {
	    select.accept(0, _laddMessage);
	    select.accept(1, _lsetReceiver, 2);
	    select.accept(2, _lclear, 3);
	    select.accept(3, _lshutDown, 4);
	    select.accept(4, _lsetActionTrigger, 2);
	    _CALLS=selectActionTriggers()+_STATIC_CALLS;
	    for(int i=nbOfMessages-1;i>=0;i--){
                if((i>0)&&(receivers.length == 0)){
                  i=1; //write only the first message
                  continue;
                }
		target=getTypeIndex((Message)messages.get(i));
		if(target>=0){
		    //MySystem.out.println("OneMessageQueue:"+name+":target declared:");
		    int k=procedures[target];
		    if(k>0){
			//MySystem.out.println("OneMessageQueue:"+name+":procedure:"+k);
			select.call(i+_CALLS, receivers[target], k);
			select.call(i+_CALLS+MessageQueue.REFUSE, receivers[target], k+MessageQueue.REFUSE);
		    }else{
			//identifying the message to send as k
			select.call(i+_CALLS, receivers[target], receivers[target]._getMessage);
			select.call(i+_CALLS+MessageQueue.REFUSE, receivers[target], receivers[target]._getMessage+MessageQueue.REFUSE);
		    }
		}else{ // default
		    if(types.length>0){
			//MySystem.out.print("OneMessageQueue:"+name+":Default:");
			//MySystem.out.println(messages.get(i)+":::");
		    }
		    if(receiver != null){
			select.call(i+_CALLS, receiver, receiver._getMessage);
			select.call(i+_CALLS+MessageQueue.REFUSE, receiver, receiver._getMessage+MessageQueue.REFUSE);
		    }
		}
	    }
	    //if(nbOfMessages > 0 && receiver != null) select.call(4, receiver, receiver._getMessage, 1);

	    if(traced)
		MySystem.out.println("OneMessageQueue:Before reply");
	    reply = Synch_Wait()-_CALLS;
	    if(traced)
		MySystem.out.println("OneMessageQueue:After reply:"+reply);

	    if((reply<0)&&(reply+_CALLS-_STATIC_CALLS>=0)){
		choiceActionTrigger(reply+_CALLS-_STATIC_CALLS);
	    }
	    if(reply >= 0) {
		boolean refused=false;
		if(reply>=MessageQueue.REFUSE){
		    refused=true;
		    reply -= MessageQueue.REFUSE;
		}
		MessageReceiver mReceiver;
		target=getTypeIndex((Message)messages.get(reply));
		if(target>=0)
		    mReceiver=receivers[target];
		else
		    mReceiver=receiver;
		Message chosen=(Message)messages.remove(reply);
		nbOfMessages--;
		if(!refused)
		    mReceiver.getMessage(chosen);
		else
		    mReceiver.Synch_Reply();
/*
		if(target>=0){
		    for(int k=nbOfMessages-1;k>=0;k--){
			//MySystem.out.println("OneMessageQueue:messages number:"+nbOfMessages);
			if(types[target].accepts((Message_Command)messages.get(k))){
			    nbOfMessages--;
			    mReceiver.getMessage((Message)messages.remove(k));
			}
		    }
		}
*/
		//mReceiver.Synch_Reply();
	    }
	}
    }
    public String toString(){
	return "OneMessageQueue:"+name;
    }
}

/**
 * This class stores messages on multiple queues.
 *
 * @author Michel Galley, Marius Silaghi
 * @see MessageReceiver
 * @see GenericAgent
 * @see Protocol
 * @see MessageSender
 * @version 1.0 00/02/04
 */

public class MessageQueue implements Runnable, Reply{
    public String name;
    private OneMessageQueue[] queues;
    private Thread[] queueThreads;
    private Message message = null;
    private MessageReceiver receiver = null;
    private boolean shuttingDown = false;
    private int procedureID;
    private ActionTrigger actionTrigger;
    private boolean hasActivators = false;

    public static final int REFUSE = 10000;
    public final int _addMessage = 1;
    public final int _setReceiver = 2;
    public final int _clear = 3;
    public final int _shutDown = 4;
    public final int _hasShutDown = 5;
    public final int _setReceiverType = 6;
    public final int _setReceiverTypeAndProcedure = 7;
    public final int _setActionTrigger = 8;


 /**    active object support (needs implements Reply)               **/
 /*********************************************************************/
 /* Written by Claude Petitpierre,   LTI-DI-EPFL,           7 Apr 1999*/
 /*********************************************************************/
 /* Modif. by Marius-Calin Silaghi, LIA-DI-EPFL,silaghi@lia.di.epfl.ch*/
 /*  Bug that may lead to deadlock in Synch_Wait(),fixed,  30 Nov 1999*/
 /*********************************************************************/
 /**/ Synch_List select = new Synch_List(this) ;                      //
 /**/ synchronized public void Synch_Reply()                          //
 /**/   { notify(); }                                                 //
 /**/ int now() {return Kernel.kern.currentTime;}                     //
 /**/ int Synch_Wait() {                                              //
 /**/   Kernel.kern.Synch_Request(select);                            //
 /**/   do                                                            //
 /**/     {try {wait();} catch(InterruptedException e){}}             //
 /**/   while(select.nbEntries!=0);                                   //
 /**/   return select.Value; }                                        //
 /*********************** Need not be modified ************************/


    public MessageQueue(int queueSize, MessageReceiver receiver, String name) {
	this.name=name;
	queues = new OneMessageQueue[queueSize];
	queueThreads = new Thread[queueSize];
	for(int i=0; i<queueSize; i++) {
	    queues[i] = new OneMessageQueue(receiver,name);
	    queueThreads[i] = new Thread(queues[i]);
	    queueThreads[i].start();
	}
    }

    public boolean hasActivators(){
	return hasActivators;
    }
    public int size(){
	int sum=0;
	for(int k=0;k<queues.length;k++)
	    sum+=queues[k].getNbOfMessages();
	return sum;
    }

    public void printMessages(){
	for(int k=0;k<queues.length;k++)
	    queues[k].printMessages();
    }

    synchronized public void addMessage(Message message){
	this.message = message;
	Synch_Reply();
    }

    // This method must not be called during a computation:
    private MessageSelector crtType;

    public void setActionTrigger(ActionTrigger at){
	actionTrigger=at;
        hasActivators=true;
	Synch_Reply();
    }

   synchronized public void setReceiverType(MessageReceiver receiver, MessageSelector type){
	this.receiver = receiver;
	this.crtType = type;
	Synch_Reply();
    }

    synchronized public void setReceiverType(MessageReceiver receiver, MessageSelector type, int procedureID){
	this.receiver = receiver;
	this.crtType = type;
	this.procedureID = procedureID;
	Synch_Reply();
    }

    // This method must not be called during a computation:
    synchronized public void setReceiver(MessageReceiver receiver){
	this.receiver = receiver;
	Synch_Reply();
    }

    // warning : the queue isn't cleared immediately
    synchronized public void clear() { hasActivators = false; Synch_Reply(); }

    synchronized public void shutDown(){
	shuttingDown = false;
	Synch_Reply();
    }

    synchronized public boolean hasShutDown(){
	for(int i=0; i<queueThreads.length; i++)
	    if(queueThreads[i].isAlive()) {
		Synch_Reply();
		return false;
	    }
	Synch_Reply();
	return true;
    }
    boolean traced=false;
    synchronized public void run(){
	select.thread=Thread.currentThread();
	int reply;
	if(name.equals("protocolInQueue"))
	    traced=false;
	for(;;) {

	    if(!shuttingDown)
		select.accept(0, _addMessage);
	    select.accept(1, _setReceiver, 1);
	    select.accept(20, _setReceiverType, 1);
	    select.accept(21, _setReceiverTypeAndProcedure, 1);
	    select.accept(22, _setActionTrigger, 1);
	    select.accept(2, _clear, 2);
	    select.accept(3, _shutDown, 3);
	    select.accept(4, _hasShutDown, 3);

	    if(traced)
		MySystem.out.println("MessageQueue:BeforeReply:"+name);
	    reply = Synch_Wait();
	    if(traced)
		MySystem.out.println("MessageQueue:AfterReply:"+name+":"+reply);
	    switch(reply) {
	    case 0: // addMessage
                //MySystem.out.println("MessageQueue:got:"+message);
		for(int index = 0; index<queues.length; index++)
		    select.call(index, queues[index],
				queues[index]._laddMessage);

		if(traced)
		    MySystem.out.println("MessageQueue:beforeSetMessage:"+name);
		reply = Synch_Wait();
		if(traced)
		    MySystem.out.println("MessageQueue:messageSet:"+name);
		queues[reply].laddMessage(message);
		break;
	    case 1: // setReceiver
		for(int index = 0; index<queues.length; index++) {
		    select.call(0, queues[index], queues[index]._lsetReceiver);
		    Synch_Wait();
		    queues[index].lsetReceiver(receiver);
		}
		break;
	    case 20: // setReceiverType
		for(int index = 0; index<queues.length; index++) {
		    select.call(0, queues[index], queues[index]._lsetReceiver);
		    Synch_Wait();
		    queues[index].lsetReceiver(receiver,crtType);
		    //queues[index].Synch_Reply();

		}
		break;
	    case 21: // setReceiverTypeAndProcedure
		for(int index = 0; index<queues.length; index++) {
		    select.call(0, queues[index], queues[index]._lsetReceiver);
		    Synch_Wait();
		    queues[index].lsetReceiver(receiver,crtType,procedureID);
		    //queues[index].Synch_Reply();
		}
		break;
	    case 22: // setActionTrigger
		for(int index = 0; index<queues.length; index++) {
		    select.call(0, queues[index], queues[index]._lsetActionTrigger);
		    Synch_Wait();
		    queues[index].lsetActionTrigger(actionTrigger);
		    //queues[index].Synch_Reply();
		}
		break;
	    case 2: //clear
		//MySystem.err.println("MessageQueue:"+name+":CLEARS");
		for(int i=0; i<queues.length; i++) {
		    select.call(0, queues[i], queues[i]._lclear);
		    Synch_Wait();
		    queues[i].lclear();
		}
		break;
	    case 3: //_shutDown
		for(int i=0; i<queues.length; i++) {
		    select.call(0, queues[i], queues[i]._lshutDown);
		    Synch_Wait();
		    queues[i].lshutDown();
		}
	    }
	}
    }
    public String toString(){
	return "MessageQueue:"+name;
    }
}
