/*
The package platform is a semester project by Galley, Michel,
that Silaghi, Marius-Calin adviced in Spring 2000.
Several modifications were done subsequently by Silaghi, Marius-Calin.
Copyright (c) 2000, 2001, 2002, 2004
*/
/**
 * @(#)MessageSender.java  1.0 00/02/04
 *
 * Intelligent Agents Collaborating in Distributed Problem Solving
 *
 * Supervisors:   Marius-Calin Silaghi, Djamila Sam-Haroud
 * Author:        Michel Galley
 */

package platform;

import sjavap.*;
import java.lang.*;
import java.net.*;
import java.io.*;
import java.util.*;
import utils.*;

/**
 * The only purpose of this class is to continuously send messages it
 * receives to other agents. When there is no message to send, it enters
 * in a wait state. The messages are given by the outQueue object,
 * through getMessage calls.
 * <p>To send a message, it open a TCP connection with the message's
 * destination server, and it uses an object stream to transmit the message.
 *
 * @author Michel Galley
 * @see Server
 * @see MessageQueue
 * @version 1.0 00/02/04
 */

class MessageSender implements Runnable, Reply, MessageReceiver{
    GenericAgent genericAgent;

 /**    active object support (needs implements Reply)               **/
 /*********************************************************************/
 /* Written by Claude Petitpierre,   LTI-DI-EPFL,           7 Apr 1999*/
 /*********************************************************************/
 /* Modif. by Marius-Calin Silaghi, LIA-DI-EPFL,silaghi@lia.di.epfl.ch*/
 /*  Bug that may lead to deadlock in Synch_Wait(),fixed,  30 Nov 1999*/
 /*********************************************************************/
 /**/ Synch_List select = new Synch_List(this) ;                      //
 /**/ synchronized public void Synch_Reply()                          //
 /**/   { notify(); }                                                 //
 /**/ int now() {return Kernel.kern.currentTime;}                     //
 /**/ int Synch_Wait() {                                              //
 /**/   Kernel.kern.Synch_Request(select);                            //
 /**/   do                                                            //
 /**/     {try {wait();} catch(InterruptedException e){}}             //
 /**/   while(select.nbEntries!=0);                                   //
 /**/   return select.Value; }                                        //
 /*********************** Need not be modified ************************/

    boolean  end=false;
    Message sentMessage;

    /** MessageSender's run method. */
    synchronized public void run(){
	select.thread=Thread.currentThread();
	for(;!end;) {
	    select.accept(0, _getMessage);
	    //System.out.println("MessageSender: Synch_Wait()...");
	    Synch_Wait();
	    //System.out.println("MessageSender: ok.");
	    sendMessage();
	    sentMessage=null;
	}
    }

    String getReceiverName(){
	if(sentMessage==null)
	    return null;
	return sentMessage.getReceiverID().getName();
    }

    /****************
     * Public methods
     ****************/

     MessageSender(GenericAgent ga){
	 genericAgent = ga;
     }

    /** sJava identification for the shutDown method. */
    public final static int _getMessage = 1;
    /**
     * Get a new message and send it to its destination.
     *
     * @param  sentMessage The message to send
     */
    synchronized public void getMessage(Message _sentMessage){
	sentMessage=_sentMessage;
	Synch_Reply();
    }

    synchronized private void sendMessage(){
	//MySystem.out.println(sentMessage.getMessageNum());
	//MySystem.out.flush();
	InetAddress dest_host = sentMessage.getReceiverID().getHost();
	int dest_port = sentMessage.getReceiverID().getPort();

	Socket socket;
	ObjectInputStream in;
	ObjectOutputStream output;
	boolean sent = false;

	//MySystem.err.println("Want to send "+sentMessage);
	while(!sent) {
            //System.out.println("MessageServer: try to send:"+sentMessage);
	    try {
		// Open a new socket connection
		socket = new Socket(dest_host, dest_port);
		socket.setTcpNoDelay(true);
		// Open a new object stream
		output = new ObjectOutputStream(socket.getOutputStream());
		// Transmit the message
		output.writeObject(sentMessage);
		//MySystem.err.println("Message sent.Try to get ack from "+sentMessage.getReceiverID().toString());
		//MySystem.err.flush();
		// Close all
		if(Server.acknowledging){
		    Object obj;
		    Integer res=null;
		    try{
			in  = new ObjectInputStream(socket.getInputStream());
			obj = in.readObject();
			//MySystem.err.println("ack Got");
			try{
			    res = (Integer)obj;
			}catch(Exception ex){
			    if(end)
				return;
			    ex.printStackTrace();
			    MySystem.err.println("When sending "+sentMessage);
			    MySystem.err.println("Instead of Integer, got "+obj);
			    if(sentMessage==obj){
				MySystem.err.println("Got Sent");
			    }else
				MySystem.err.println("Got different from Sent");
			    output.close();
			    socket.close();
			    //???continue;
			    MySystem.exit(1);
			}
		    }catch(Exception e){
                        if(end)
                          return;
			if(res!=null)
			    System.err.println("MessageSender:Result "+res);
			e.printStackTrace();
			output.close();
			socket.close();
			//???continue;
			MySystem.exit(1);
		    }
		}
		output.close();
		//in.close();
		socket.close();
	    }catch(IOException e){
		continue;  /* Can't connect : will try later */ }
	    //MySystem.err.println("Sender did sent "+sentMessage);

            sent = true;
	    // Maybe send a copy of the message to the broker if
	    // the message went to another agent
	    if((Agent.notifying)&&(genericAgent instanceof Agent)&&
 	        (!sentMessage.getReceiverID().equals(((Agent)genericAgent).getBrokerID()))&&
                (sentMessage instanceof Message_Command)){
	        Message_Command command = (Message_Command)sentMessage;
	        Message_NotifyMessage notify = new Message_NotifyMessage
		    (command.getCommandStr(),
		     command.getArgumentsStr(),
		     command.getReceiverID().toString(),
		     command.getSenderID().toString(),
		     (short)command.getComputationNum(),
		     command.getMessageNum(),
		     ((Agent)genericAgent).getBrokerID());
                notify.setOtherData(command.getSenderID(),
                                    command.getComputationNum(),
                                    command.getMessageNum());
                sentMessage = notify;
	        dest_host = sentMessage.getReceiverID().getHost();
	        dest_port = sentMessage.getReceiverID().getPort();
                sent=false;
	    }
	    //MySystem.err.println("Sender did notify "+sentMessage);
	}


	// SHOW SENT MESSAGES
	//if(!(sentMessage instanceof Message_Command)) {
	//System.out.println(sentMessage);
	//}
	// SHOW SENT MESSAGES
	//if(!(sentMessage instanceof Message_Command)) {
	//System.out.println(new MessageLog(sentMessage));
	//}
    }
}
