/*
The package platform is a semester project by Galley, Michel,
that Silaghi, Marius-Calin adviced in Spring 2000.
Several modifications were done subsequently by Silaghi, Marius-Calin.
Copyright (c) 2000, 2001, 2002, 2004
*/
/**
 * @(#)Message_Command.java  1.0 00/02/05
 *
 * Intelligent Agents Collaborating in Distributed Problem Solving
 *
 * Supervisors:   Marius-Calin Silaghi, Djamila Sam-Haroud 
 * Author:        Michel Galley
 */

package platform;

import java.io.*;

/**
 * The <code>Message_Command</code> represents a message sent by a protocol. It
 * consists of a command and its arguments. Both command and arguments are
 * serializable objects.
 *
 * @author Michel Galley
 * @see Message
 * @version 1.0 00/02/05 
 */

public class Message_Command extends Message
{
    /** @serial The command. */
    private Serializable command;
    /** @serial Command's argument. */
    private Serializable arguments;
    
    /** 
     * Initializes a newly created Message_Command object. This constructor
     * simply calls it's superclass constructor.
     */
    public Message_Command() { super(); }
    
    /**
     *  Initializes a newly created Message_Command object.
     *
     * @param  command Command sent to the receiver
     * @param  arguments Arguments of the command
     * @param  receiverID Receiver of the command
     */
    public Message_Command(Serializable command, Serializable arguments, 
			   AgentID receiverID)
    {
	super(receiverID);
	this.command = command;
	this.arguments = arguments;
    }
    
    /** Get the command. */
    public Serializable getCommand() { return command; }
    /** Get the arguments. */ 
    public Serializable getArguments() { return arguments; }
    public void setArguments(Serializable arg) { arguments = arg; }

    /** Get a string representation of the command. */
    String getCommandStr()
    {
	if(command != null)
	    return command.toString();
	return "";
    }

    /** 
     * Get a string representation of the arguments. If the arguments 
     * consist of an array, the content of array is shown. 
     */
    String getArgumentsStr()
    {
	String str = "";
	if(arguments != null) {
	    try {
		Object[] array = (Object[])arguments;
		for(int i=0; i<array.length; i++)
		    str += array[i].toString() + " "; 
	    }
	    catch(ClassCastException e) {
		// Not an array
		str += arguments.toString() + " "; 
	    }
	}		
	return str;
    }

    /** Gives a string representation of the command and its arguments. */
    String _toString() 
    { return getCommandStr() + " " + getArgumentsStr(); }
    /**
     * writes Message_Command's specific member variables on a 
     * DataOutputStream.
     *
     * @param  out The DataOutputStream to write on 
     */
    void _writeMessage(DataOutputStream out) throws IOException 
    {
	String commandStr = getCommandStr();
	out.writeShort(commandStr.length());  //2
	out.writeBytes(commandStr);
	String argsStr = getArgumentsStr();   
	out.writeShort(argsStr.length());     //2
	out.writeBytes(argsStr);
    }
    /**
     * Gives the size in bytes of the data written on a DataOutputStream 
     * when calling _writeMessage. 
     */
    int _size() 
    { 
	 return 4 + getCommandStr().length() + getArgumentsStr().length();
    }
    /**
     * This method reads Message_Command's specific member variables 
     * from a DataInputStream. There is a deserialization limitation: command 
     * and arguments are converted to String.
     *
     * @param in The DataInputStream to read on
     */
    void _readMessage(DataInputStream in) throws IOException 
    {
	// !!! Deserialization limitation: command and arguments 
	// are converted to String !!!
	// But we don't need these two objects after having put them
	// in a file.

	short size = in.readShort();     //2
	byte[] str = new byte[size];
	in.readFully(str);
	command = new String(str);
	size = in.readShort();           //2
	str = new byte[size];
	in.readFully(str);
	arguments = new String(str);
    }

}    
