/*
The package platform is a semester project by Galley, Michel,
that Silaghi, Marius-Calin adviced in Spring 2000.
Several modifications were done subsequently by Silaghi, Marius-Calin.
Copyright (c) 2000, 2001, 2002, 2004
*/
/**
 * @(#)Message_CreateColorMatrix.java  1.0 00/02/05
 *
 * Intelligent Agents Collaborating in Distributed Problem Solving
 *
 * Supervisors:   Marius-Calin Silaghi, Djamila Sam-Haroud
 * Author:        Michel Galley
 */

package platform;

import csp.*;
import java.io.*;
import java.awt.*;
import java.awt.geom.*;

/**
 * <code>Message_CreateColorMatrix</code> messages are sent by a protocol to
 * ask the broker to create a new Matrix representation on its GUI. Please
 * refer to the Platform or Agent classes for further explainations.
 *
 * @author Michel Galley
 * @see Message
 * @see Platform#createColorMatrix
 * @see Agent#createColorMatrix
 * @version 1.0 00/02/05
 */

public class Message_CreateColorMatrix extends Message{
    /** @serial The first variable name. */
    private String var1;
    /** @serial The second variable name. */
    private String var2;
    /** @serial Matrix's width. */
    private int sizeX;
    /** @serial Matrix's height. */
    private int sizeY;
    public int id=0;
    /**
     * Initializes a newly created Message_CreateColorMatrix object. It is
     * used to send a request to the broker to create a matrix representing
     * the constraint shared by two variables (this matrix is shown in the
     * GUI). This constructor simply calls it's superclass constructor.
     */
    public Message_CreateColorMatrix() { super(); }

    /**
     * Initializes a newly created Message_CreateColorMatrix object. It is
     * used to send a request to the broker to create a matrix representing
     * the constraint shared by two variables (this matrix is shown in the
     * GUI).
     *
     * @param var1 The first variable
     * @param var1 The second variable
     * @param sizeX The width of the matrix
     * @param sizeY The height of the matrix
     * @param brokerID The ID of the broker
     */
    public Message_CreateColorMatrix(int id, String var1, String var2,
				     int sizeX, int sizeY,
				     AgentID brokerID){
	super(brokerID);
        this.id = id;
	this.var1 = var1;
	this.var2 = var2;
	this.sizeX = sizeX;
	this.sizeY = sizeY;
    }

    /** Get the first variable. */
    public String getVariable1() { return var1; }
    /** Get the second variable. */
    public String getVariable2() { return var2; }
    /** Get the width of the matrix. */
    public int getSizeX() { return sizeX; }
    /** Get the height of the matrix. */
    public int getSizeY() { return sizeY; }

    String _toString()
    {
	return "CreateColorMatrix vars: "+var1+","+var2
	  +" dim: "+sizeX+"x"+sizeY;
    }

    void _writeMessage(DataOutputStream out) throws IOException
    {
	out.writeShort(var1.length());
	out.writeBytes(var1);
	out.writeShort(var2.length());
	out.writeBytes(var2);
	out.writeShort(sizeX);
	out.writeShort(sizeY);
    }
    int _size()
    {
	return 8 + var1.length() + var2.length();
    }
    void _readMessage(DataInputStream in) throws IOException
    {
	short size = in.readShort();
	byte[] str = new byte[size];
	in.readFully(str);
	var1 = new String(str);
	size = in.readShort();
	str = new byte[size];
	in.readFully(str);
	var2 = new String(str);
	sizeX = in.readShort();
	sizeY = in.readShort();
    }
}
