/*
The package platform is a semester project by Galley, Michel,
that Silaghi, Marius-Calin adviced in Spring 2000.
Several modifications were done subsequently by Silaghi, Marius-Calin.
Copyright (c) 2000, 2001, 2002, 2004
*/
/**
 * @(#)Message_NotifyMessage.java  1.0 00/02/23
 *
 * Intelligent Agents Collaborating in Distributed Problem Solving
 *
 * Supervisors:   Marius-Calin Silaghi, Djamila Sam-Haroud
 * Author:        Michel Galley, Marius Silaghi
 */

package platform;

import java.io.*;

/**
 * Platform-dependent message sent by agents to the broker to notify him
 * about messages they have received from other agents. The only messages
 * that has to be notified are protocol-dependent messages (agents don't
 * send platform messages to each other).
 * <p>By default, these messages are not sent because this might double the
 * number of messages.
 *
 * @author Michel Galley, Marius Silaghi
 * @see Message
 * @version 1.0 00/02/23
 */

class Message_NotifyMessage extends Message
{
    private String commandText;
    private String argumentsText;
    String origSenderID, origReceiverID;
    short origComputationNum;
    int origMessageNum;

    Message_NotifyMessage() { super(); }
    /**
     *  Initializes a newly created Message_NotifyMessage object.
     *
     * @param  commandText Command sent to the receiver
     * @param  argumentsText Arguments of the command
     * @param  origSenderID The ID of the agent that sent the original message
     * @param  receiverID Receiver of this message (the broker)
     */
    Message_NotifyMessage(String commandText, String argumentsText,
                          String origReceiverID,
			  String origSenderID, short origComputationNum,
			  int origMessageNum, AgentID brokerID)
    {
	super(brokerID);
	this.commandText = commandText;
	this.argumentsText = argumentsText;
	this.origSenderID = origSenderID;
	this.origReceiverID = origReceiverID;
	this.origComputationNum = origComputationNum;
	this.origMessageNum = origMessageNum;
    }

    /** Get the ID of the agent that has sent the original message */
    String getOrigSenderID() { return origSenderID; }
    /** Get the ID of the agent that has sent the original message */
    String getOrigReceiverID() { return origReceiverID; }
    /** Get the computation number of the original message */
    int getOrigComutationNum() { return origComputationNum; }
    /** Get the message number of the original message */
    int getOrigMessageNum() { return origMessageNum; }
    /** A string representing the original command. */
    String getCommandStr() { return commandText; }
    /** A string representing the arguments of the original command. */
    String getArgumentsStr() { return argumentsText; }

    /** Gives a string representation of the command and its arguments. */
    public String _toString()
    { return ((getOrigSenderID().equals(getSenderID().toString()))?"S:":"R:")+
	  getCommandStr()+" "+getArgumentsStr(); }
    /**
     * writes Message_NotifyMessage's specific member variables on a
     * DataOutputStream.
     *
     * @param  out The DataOutputStream to write on
     */
    public void _writeMessage(DataOutputStream out) throws IOException
    {

	String commandStr = getCommandStr();
	out.writeShort(commandStr.length());  //2
	out.writeBytes(commandStr);
	String argsStr = getArgumentsStr();
	out.writeShort(argsStr.length());     //2
	out.writeBytes(argsStr);
	String agentStr = getOrigSenderID();
	out.writeShort(agentStr.length());    //2
	out.writeBytes(agentStr);
	agentStr = getOrigReceiverID();
	out.writeShort(agentStr.length());    //2
	out.writeBytes(agentStr);
	out.writeShort(origComputationNum);   //2
	out.writeInt(origMessageNum);         //4
    }
    /**
     * Gives the size in bytes of the data written on a DataOutputStream
     * when calling _writeMessage.
     */
    public int _size()
    {
	return 14 + getCommandStr().length()
	    + getArgumentsStr().length()
	    + getOrigSenderID().length()
	    + getOrigReceiverID().length();
    }
    /**
     * This method reads Message_NotifyMessage's specific member variables
     * from a DataInputStream. There is a deserialization limitation: command
     * and arguments are converted to String.
     *
     * @param in The DataInputStream to read on
     */
    public void _readMessage(DataInputStream in) throws IOException
    {
	short size = in.readShort();         //2
	byte[] str = new byte[size];
	in.readFully(str);
	commandText = new String(str);

	size = in.readShort();               //2
	str = new byte[size];
	in.readFully(str);
	argumentsText = new String(str);

	size = in.readShort();               //2
	str = new byte[size];
	in.readFully(str);
	origSenderID = new String(str);

	size = in.readShort();               //2
	str = new byte[size];
	in.readFully(str);
	origReceiverID = new String(str);

	origComputationNum = in.readShort(); //2
	origMessageNum = in.readInt();       //4
    }
}
