/*
The package platform is a semester project by Galley, Michel,
that Silaghi, Marius-Calin adviced in Spring 2000.
Several modifications were done subsequently by Silaghi, Marius-Calin.
Copyright (c) 2000, 2001, 2002, 2004
*/
/**
 * @(#)Message_SetColor.java  1.0 00/02/05
 *
 * Intelligent Agents Collaborating in Distributed Problem Solving
 *
 * Supervisors:   Marius-Calin Silaghi, Djamila Sam-Haroud
 * Author:        Michel Galley, Marius-Calin Silaghi
 */

package platform;

import java.io.*;
import java.awt.*;
import java.awt.geom.*;
import utils.*;
import csp.*;

/**
 * <code>Message_SetColor</code> message is sent by a protocol to ask
 * the broker to put a new color in the cell positioned at (posX, posY) in the
 * broker GUI's matrix involving both var1 and var2 variables. Please
 * refer to the Platform or Agent classes for further explainations.
 *
 * @author Michel Galley, Marius-Calin Silaghi
 * @see Message
 * @see Platform#createColorMatrix
 * @see Agent#createColorMatrix
 * @version 1.0 00/02/05
 */

public class Message_SetColor extends Message {
    /** @serial The first variable name. */
    private String var1;
    /** @serial The second variable name. */
    private String var2;
    /** @serial The x coordinate on the matrix. */
    private int posX;
    /** @serial The y coordinate on the matrix. */
    private int posY;
    /** @serial The x2 coordinate on the matrix. */
    private int posX2;
    /** @serial The y2 coordinate on the matrix. */
    private int posY2;
    Range[] domain,oldDomain;
    int table[][]=null;
    int destX,destY;
    /** @serial The new color set at the (x, y) coordinate. */
    private Color color, color0;
    /** @serial The old color at the (x, y) coordinate. */
    private Color oldColor, oldColor0;

    public Message_SetColor() { super(); }
    /**
     * Put a new color in the cell positioned at (posX, posY) in the
     * matrix (of the GUI) involving both var1 and var2 variables.
     *
     * @param  var1 Name of the first variable
     * @param  var2 Name of the second variable
     * @param  sizeX Width of the matrix
     * @param  sizeY Height of the matrix
     * @param  color The new color of the cell
     * @param  oldColor The old color of the cell
     * @param brokerID The ID of the broker
     */
    public Message_SetColor(String var1, String var2,
			    int posX, int posY,
			    Color color, Color oldColor,
			    AgentID brokerID){
	super(brokerID);
	this.var1 = var1;
	this.var2 = var2;
	this.posX = posX;
	this.posY = posY;
	this.posX2 = posX;
	this.posY2 = posY;
	this.color = color;
	this.oldColor = oldColor;
    }
    public Message_SetColor(String var1, String var2,
			    Range[] domain,
			    Range[] oldDomain,
			    Color color, Color oldColor,
			    AgentID brokerID){
	super(brokerID);
	this.var1 = var1;
	this.var2 = var2;
	this.domain = domain;
	this.oldDomain = oldDomain;
	this.color = color;
	this.oldColor = oldColor;
    }
    /**
     * Put a new color in the cell positioned at (posX, posY) in the
     * matrix (of the GUI) involving both var1 and var2 variables.
     *
     * @param  var1 Name of the first variable
     * @param  var2 Name of the second variable
     * @param  sizeX Width of the matrix
     * @param  sizeY Height of the matrix
     * @param  color The new color of the 1 cell
     * @param  oldColor The old color of the 1 cell
     * @param  color0 The new color of the 0 cell
     * @param  oldColor0 The old color of the 0 cell
     * @param brokerID The ID of the broker
     */
    public Message_SetColor(String var1, String var2,
			    int posX, int posY,
                            int posX2, int posY2,
			    int table[][],
                            int destX, int destY,
			    Color color, Color oldColor,
			    Color color0, Color oldColor0,
			    AgentID brokerID){
	super(brokerID);
	this.var1 = var1;
	this.var2 = var2;
	this.posX = posX;
	this.posY = posY;
	this.posX2 = posX2;
	this.posY2 = posY2;
	this.table = table;
	this.destX = destX;
	this.destY = destY;
	this.color = color;
	this.oldColor = oldColor;
	this.color0 = color0;
	this.oldColor0 = oldColor0;
    }

    /**
     * Put a new color in the cell positioned at (posX, posY) in the
     * matrix (of the GUI) involving both var1 and var2 variables.
     *
     * @param  var1 Name of the first variable
     * @param  var2 Name of the second variable
     * @param  sizeX Width of the matrix
     * @param  sizeY Height of the matrix
     * @param  color The new color of the cell
     * @param  oldColor The old color of the cell
     * @param brokerID The ID of the broker
     */
    public Message_SetColor(String var1, String var2,
			    int posX, int posY,
                            int posX2, int posY2,
			    Color color, Color oldColor,
			    AgentID brokerID) {
	super(brokerID);
	this.var1 = var1;
	this.var2 = var2;
	this.posX = posX;
	this.posY = posY;
	this.posX2 = posX2;
	this.posY2 = posY2;
	this.color = color;
	this.oldColor = oldColor;
    }

    /** Return the first variable name. */
    public String getVariable1() { return var1; }
    /** Return the second variable name. */
    public String getVariable2() { return var2; }
    /** Return the X coordinate of this object. */
    public int getPosX() { return posX; }
    /** Return the Y coordinate of this object. */
    public int getPosY() { return posY; }
    /** Return the X2 coordinate of this object. */
    public int getPosX2() { return posX2; }
    /** Return the Y2 coordinate of this object. */
    public int getPosY2() { return posY2; }
    /** Return the new color set at the (x, y) coordinate. */
    public Color getColor() { return color; }
    public Color getColor0() { return color0; }
    /**
     * Return the old color found at the (x, y) coordinate
     * before the new one is set.
     */
    public Color getOldColor() { return oldColor; }
    public Color getOldColor0() { return oldColor0; }
    /**
     * Return a string representation of the variables, the coordinates,
     * and the colors defined in this object.
     */
    String _toString()
    {
	return "SetColor vars: "+var1+","+var2
       +((domain==null)?((table==null)?
       (" at: ("+posX+"-"+posX2+","+posY+"-"+posY2+")"):" table"):" domains "+domain[0]+domain[1])
       +" color: ["+color.getRed()+","+color.getGreen()+","+color.getBlue()+"]"
       +" old color: ["+oldColor.getRed()+","+oldColor.getGreen()+","
	              +oldColor.getBlue()+"]";
    }
    /**
     * writes Message_SetColor's specific member variables on a
     * DataOutputStream.
     *
     * @param  out The DataOutputStream to write on
     */
    void _writeMessage(DataOutputStream out) throws IOException{
	if(domain!=null)
	    out.writeShort(Utils.nullElement);              //2
	out.writeShort(var1.length());              //2
	out.writeBytes(var1);
	out.writeShort(var2.length());              //2
	out.writeBytes(var2);
	if(domain==null){
	    out.writeShort((short)posX);              //2
	    out.writeShort((short)posY);              //2
	    out.writeShort((short)posX2);              //2
	    out.writeShort((short)posY2);              //2
	}else{
	    out.writeShort((short)domain[0].getL());              //2
	    out.writeShort((short)domain[0].getU());              //2
	    out.writeShort((short)domain[1].getL());              //2
	    out.writeShort((short)domain[1].getU());              //2
	    if(oldDomain!=null){
		out.writeShort((short)oldDomain[0].getL());
		out.writeShort((short)oldDomain[0].getU());
		out.writeShort((short)oldDomain[1].getL());
		out.writeShort((short)oldDomain[1].getU());
	    }else
		out.writeShort(Utils.nullElement);
	}
	out.writeShort(color.getRed());              //2
	out.writeShort(color.getGreen());              //2
	out.writeShort(color.getBlue());              //2
	out.writeShort(oldColor.getRed());              //2
	out.writeShort(oldColor.getGreen());              //2
	out.writeShort(oldColor.getBlue());              //2
	//table
	if(table==null){
	    out.writeShort(0);              //2
	    return;
	}
	//out.writeShort(1);
	out.writeShort(table.length);              //2
	if(table.length==0) {
	    MySystem.exit(1);
	    return;
	}
	out.writeShort(table[0].length);              //2
	for(int i=0;i<table.length;i++)
	    for(int j=0;j<table[0].length;j++)
		out.writeShort(table[i][j]);              
	out.writeShort((short)destX);              //2
	out.writeShort((short)destY);              //2
	out.writeShort(color0.getRed());              //2
	out.writeShort(color0.getGreen());              //2
	out.writeShort(color0.getBlue());              //2
	out.writeShort(oldColor0.getRed());              //2
	out.writeShort(oldColor0.getGreen());              //2
	out.writeShort(oldColor0.getBlue());              //2
    }
    /**
     * Gives the size in bytes of the data written on a DataOutputStream
     * when calling _writeMessage.
     */
    int _size() { 
	int dim= 4 + 12+
	    var1.length() + 
	    var2.length() +
	    8+((domain!=null)?(2+((oldDomain!=null)?8:2)):0) +
	    2+((table==null)?0:18);
	if((table!=null)&&(table.length>0)) 
	    dim+=table.length*table[0].length*2;
	return dim;
    }
    /**
     * This method reads Message_SetColor's specific member variables
     * from a DataInputStream.
     *
     * @param in The DataInputStream to read on
     */
    void _readMessage(DataInputStream in) throws IOException{
	domain = null;
	short size = in.readShort();
	if(size==Utils.nullElement){
	    domain=new Range[2];
	    size = in.readShort();
	}
	byte[] str = new byte[size];
	in.readFully(str);
	var1 = new String(str);
	size = in.readShort();
	str = new byte[size];
	in.readFully(str);
	var2 = new String(str);
	if(domain==null){
	    posX = in.readShort();
	    posY = in.readShort();
	    posX2 = in.readShort();
	    posY2 = in.readShort();
	}else{
	    int l,u;
	    l = in.readShort();
	    u = in.readShort();
	    domain[0]=new Range(l,u);
	    l = in.readShort();
	    u = in.readShort();
	    domain[1]=new Range(l,u);
	    l = in.readShort();
	    if(l!=Utils.nullElement){
		oldDomain=new Range[2];
		u = in.readShort();
		oldDomain[0] = new Range(l,u);
		l = in.readShort();
		u = in.readShort();
		oldDomain[1] = new Range(l,u);
	    }
	}
	short r = in.readShort(),  g = in.readShort(), b = in.readShort();
	color = new Color(r, g, b);
	r = in.readShort();  g = in.readShort(); b = in.readShort();
	oldColor = new Color(r, g, b);
	//table
	r = in.readShort();
	if(r==0){
	    table=null;
	    return;
	}
	g = in.readShort();
	table= new int[r][g];
	for(int i=0;i<table.length;i++)
	    for(int j=0;j<table[0].length;j++)
		table[i][j] = in.readShort();
	destX = in.readShort();
        destY = in.readShort();
	r = in.readShort();  g = in.readShort(); b = in.readShort();
	color0 = new Color(r, g, b);
	r = in.readShort();  g = in.readShort(); b = in.readShort();
	oldColor0 = new Color(r, g, b);
    }
}
