/*
The package platform is a semester project by Galley, Michel,
that Silaghi, Marius-Calin adviced in Spring 2000.
Several modifications were done subsequently by Silaghi, Marius-Calin.
Copyright (c) 2000, 2001, 2002, 2004
*/
/**
 * @(#)Message_Subscription.java  1.0 00/02/05
 *
 * Intelligent Agents Collaborating in Distributed Problem Solving
 *
 * Supervisors:   Marius-Calin Silaghi, Djamila Sam-Haroud 
 * Author:        Michel Galley
 */

package platform;

import java.io.*;

/**
 * Platform-dependent message sent by an agent to subscribe to the 
 * broker. It contains information about the protocols the agent knows.
 *
 * @author Michel Galley
 * @see Message
 * @see Message_Unsubscription
 * @version 1.0 00/02/05 
 */

class Message_Subscription extends Message{
    private Class[] protocols;    
    public Object parameters;

    Message_Subscription() { super(); }
    Message_Subscription(Class[] protocols, AgentID brokerID)
    {
	super(brokerID);
	this.protocols = protocols;
    }    

    public Class[] getProtocols()
    { return protocols; }

    public String _toString()
    {
	String str = "Subscription: ";
	
	for(int i=0; i<protocols.length; i++) {
	    try { str += protocols[i].newInstance()+" "; }
	    catch(Exception e) { e.printStackTrace(); System.exit(-1); }
	}
	return str;
    }

    public void _writeMessage(DataOutputStream out) throws IOException 
    {
	String str;
	out.writeShort(protocols.length);
	for(int i=0; i<protocols.length; i++) {
	    str = protocols[i].getName();
	    out.writeShort(str.length());
	    out.writeBytes(str);
	}
    }
    public int _size() {
	int size = 2;
	for(int i=0; i<protocols.length; i++)
	    size += 2 + protocols[i].getName().length();
	return size; 
    }
    public void _readMessage(DataInputStream in) throws IOException {
	short nb = in.readShort();
	protocols = new Class[nb];

	for(int i=0; i<nb; i++) {
	    short size = in.readShort();
	    byte[] str = new byte[size];
	    in.readFully(str);
	    try { protocols[i] = Class.forName(new String(str)); }
	    catch(ClassNotFoundException e) 
	    { e.printStackTrace(); System.exit(-1); }
	}
    }
}    
